#!/usr/bin/env python
# -*- coding: utf-8 -*-


# This file contains useful functions for dealing with the gradient controls.
# We will move them gradually from the classes files to here, for having them
# always available andeasy to use, also for future stuff..

import maya.cmds as cmds
import hmScripts.UI.hmUiCommonUtils as uiUtil
import hmScripts.UI.hmGlobalVariables as gVar

reload(uiUtil)
reload(gVar)


class GradientControl():
	'''
	This class provides useful functions for editing gradient control Attributes and No attributes
	'''
	def __init__(self, optionVarAndValues, parameterChanger):
		'''
		We need the list that we use for the option var and a boolean value for using the right command ( for the UI control or for the Node )


		optionVarAndValues(list, [optionVarName, values ])
			this is used for storing the option var and the values...

		parameterChanger(boolean)
			-Specify if the commands must be executed as parameterchanger or not (if a node is connected the beavhior must change..)

			
		'''
		self.asParameterChanger = parameterChanger
		self.opVarAndVal = optionVarAndValues



	def setNode(self, node ):
		'''
		If we call this method we can connect a node with an attribute for editing the gradient control.

		node(string)
			-node name

		'''
		# Detine wich tipe of command will be called by the class methods
		self.node = node




	def setControls(self, gradControl, interpMenu, interpPos, interpVal):
		'''
		Initialize the class providing functions for editing a gradient control

		gradControl(string, cmds.gradientControlNoAttr )
			-the gradient control to operate on

		interpMenu(string, cmds.optionMenu)
			-the option menu for selecting the interpolation

		interpPos(string, cmds.floatField)
			-the float field that set the position of the current point

		interpVal(string, cmds.floatField)
			-the float field that set the value of the current point

		'''
		self.gcProfileCurve = gradControl
		self.omInterp = interpMenu
		self.ffInterpPos = interpPos
		self.ffInterpVal = interpVal
		




	#== Profile control interp changes
	def updateGradValuesUi(self, *Args):
		'''
		Update the selector interpolation position and value of the current gradient control.
		In this way, we update the gradient shape every time something on the gradient changes.
		
		EXAMPLE USAGE:
		
		cmds.gradientControlNoAttr( ..., currentKeyChanged = gFunct.updateGradValuesUi, dragCommand = gFunct.updateGradValuesUi)



		'''
		#Gradient control get values 
		currentKey = cmds.gradientControlNoAttr(self.gcProfileCurve, q=True, currentKey=True )

		#Compile the list..
		valueList = self.getGradientControlListVal()

		#Update position and value
		cmds.floatField( self.ffInterpVal, edit=True, value= valueList[currentKey][0] )
		cmds.floatField( self.ffInterpPos, edit=True, value= valueList[currentKey][1] )
		cmds.optionMenu(self.omInterp,  edit=True, select= valueList[currentKey][2] )


	
	def cangeCurrentPoint(self, *Args):
		'''
		We query the current selected point in the ramp and we change it's interpolation, value and position according to UI.

		cmds.optionMenu( cc = gFunct.cangeCurrentPoint )
		cmds.floatField( value = 0, precision = 3, ec = gFunct.cangeCurrentPoint, cc = gFunct.cangeCurrentPoint )
		cmds.floatField( value = 0, precision = 3, ec = gFunct.cangeCurrentPoint, cc = gFunct.cangeCurrentPoint )


		'''
		if self.asParameterChanger == False: #Do it only if is a No Attr function
			#Gradient control get values 
			currentKey = cmds.gradientControlNoAttr(self.gcProfileCurve, q=True, currentKey=True ) 
			#Get ui values
			posToSet = cmds.floatField( self.ffInterpVal, q=True, v=True)
			valToSet = cmds.floatField( self.ffInterpPos, q=True, v=True)
			interpToSet = cmds.optionMenu( self.omInterp,  q=True, select=True)
			#Compile the list..
			gradientValues = self.getGradientControlListVal() 
			#Sobstitute the current key value from UI
			gradientValues[currentKey] = [posToSet, valToSet, interpToSet]
			#Do shit for converting in a suitable format for the shitty command
			flattenedList = uiUtil.flatten( gradientValues ) 
			stringToFormat = [ str(x) for x in flattenedList ]
			stringValuesToSet = ','.join(stringToFormat)

			#Set the values 
			# cmds.gradientControlNoAttr(self.gcProfileCurve, edit= True, currentKeyInterpValue = interpToSet)
			cmds.gradientControlNoAttr(self.gcProfileCurve, e=True, asString=stringValuesToSet )



	
	def getGradientControlListVal(self):
		'''
		This method use the shitty way for getting the values and format them in a usable way..
		'''
		#This shit returns all the strings
		stringVal = cmds.gradientControlNoAttr(self.gcProfileCurve, q=True, asString=True )
		stringVal = stringVal.split(',')
		
		#We'll store here the formatted values
		valueList = []
		
		#Now get the value all the keys and format them in a right way for us..
		for x in range(0, len(stringVal), 3 ):
			currValue = float( stringVal[x] )
			currPos = float( stringVal[x+1] )
			currInterp = int( stringVal[x+2] )
			valueList.append([currValue, currPos, currInterp])
			
		return valueList



	#== Gradient control preset change
	def setGradientControlTo1(self, *Args):
		self.setGradientControlTo(1)
	def setGradientControlTo2(self, *Args):
		self.setGradientControlTo(2)
	def setGradientControlTo3(self, *Args):
		self.setGradientControlTo(3)
	def setGradientControlTo4(self, *Args):
		self.setGradientControlTo(4)
	def setGradientControlTo5(self, *Args):
		self.setGradientControlTo(5)		
		
	def setGradientControlTo(self, setTo):
		'''
		This function set the gradientCurve to a preferred shape.
		'''	
		if setTo == 1:
			#Flat
			# self.changeGradientControlShape([u'0.5,0,2', u'0.5,1,2'])
			#sharp union
			self.setNewGradientShape(['0.5,0.0,3', '0.5,0.4,3', '0.5,0.5,3', '0.5,0.6,3', '0.5,1.0,3'])
			#turn off the interpolate profile
		if setTo == 2:
			#sharp 3 segment transition
			self.setNewGradientShape(['0.5,0.0,3', '0.5,0.22,3', '0.5,0.3,3', '0.5,0.38,1', '0.5,0.62,1', '0.5,0.7,3', '0.5,0.78,3', '0.5,1.0,3'])
		if setTo == 3:
			#Inside cut
			self.setNewGradientShape( ['0.5,0.0,3', '0.5,0.3,3', '0.45,0.4,3', '0.356007142633,0.45,3', '0.202898130186,0.473198335219,3', '0.000100268763501,0.5,3', '0.202898130186,0.526801664781,3', '0.356007142633,0.55,3', '0.45,0.6,3', '0.5,0.7,3', '0.5,1.0,3'])
		if setTo == 4:
			#Sharp cut
			self.setNewGradientShape(['0.5,0.0,1', '0.5,0.3,1', '0.314250139238,0.487117336351,1', '0.000100268763501,0.5,1', '0.314250139238,0.512882663649,1', '0.5,0.7,1', '0.5,1.0,1'])			
		if setTo == 5:
			#Soffietto
			self.setNewGradientShape(['0.5,0.0,2', '0.4,0.1,2', '0.5,0.2,2', '0.4,0.3,2', '0.5,0.4,2', '0.4,0.5,2', '0.5,0.6,2', '0.4,0.7,2', '0.5,0.8,2', '0.4,0.9,2', '0.5,1.0,2'])	



	def setNewGradientShape(self, listValues):
		'''
		This method will use the current provided list of values in this format: [u'0.5,0,2', u'0.5,1,2', ..., ]
		It will change the shape of the gradient control accordingly
		'''
		# Depending on the type of ui we are using, we interpret the input string in different ways
		
		#========== GRAD CONTROL CONNECTED ATTR
		if self.asParameterChanger:
			changeGradientControlShape( self.node, gVar.rampBaseId, gVar.rampVal, gVar.rampPos, gVar.rampInt, listValues)
				
		#========== GRAD CONTROL NO ATTR
		else:
			#Store the current values in the value option
			self.opVarAndVal[1] = listValues
			#Very important TRICK, let the gradient control forget about its option
			cmds.gradientControlNoAttr( self.gcProfileCurve, e=True, optionVar='' )
			#Remove the current optionVar
			cmds.optionVar( rm = self.opVarAndVal[0] )
			
			#Now for every value in the list, we build the optionVar
			for value in listValues:
				cmds.optionVar(stringValueAppend=[ self.opVarAndVal[0], value ] )
				
			#Re attach the option var to the control
			cmds.gradientControlNoAttr( self.gcProfileCurve, e=True, optionVar=self.opVarAndVal[0] )
		
	



def changeGradientControlShape( node, mainAttr, valAttr, posAttr, intAttr, listValues):
	'''
	This method will use the current provided list of values in this format: [u'0.5,0,2', u'0.5,1,2', ..., ]
	It will change the shape of the gradient control connected attribute accordingly.
	'''
	oldWay = True
	#========== GRAD CONTROL CONNECTED ATTR
	baseArrayAttribute = '%s.%s' % (node, mainAttr)
	#We count the number of points required
	requiredPoints = len(listValues) 
	#Current points 
	currentArrayLenght = cmds.getAttr( baseArrayAttribute, size=True )
	currentPoints =  cmds.getAttr( baseArrayAttribute, mi = True )

	if oldWay:
	   	# #If the array has more points than needed
		if currentArrayLenght > requiredPoints:
			pointsToRemove = currentArrayLenght - requiredPoints
			#Sort and reverse the indices list, so we will keep the smaller values
			currentPoints.sort()
			currentPoints.reverse()
			
			# print 'we remove %s points', pointsToRemove
			for x in range(pointsToRemove):
				#Make sure to remove only valid indices, otherwise we will encounter errors
				#print 'remove this index:', currentPoints[x]
				id = currentPoints[x]
				cmds.removeMultiInstance( '%s[%s]' % ( baseArrayAttribute, id ), b=True )
				
		

			
		#We compile the list with the right indices so we can use the left over points and then add newer
		currentPoints =  cmds.getAttr( baseArrayAttribute, mi = True )
		currentArrayLenght = cmds.getAttr( baseArrayAttribute, size=True )
		
		#print 'ho attualmente', currentPoints, 'l array misura: ', currentArrayLenght


		if currentArrayLenght == requiredPoints:
			indicesToOperateOn = currentPoints
			# print 'list of id:', indicesToOperateOn
			
		if requiredPoints > currentArrayLenght:	
			indicesToOperateOn = currentPoints
			pointToAdd = requiredPoints - currentArrayLenght
			for id in range(pointToAdd):
				lastId = indicesToOperateOn[len(indicesToOperateOn)-1] +1
				indicesToOperateOn.append(lastId)
			# print 'list of id:', indicesToOperateOn
			

		#print 'gli indici sui quali lavorero sono', indicesToOperateOn
		#print 'list values:', listValues
		
		# # Now, set the points as required from the input using the right ids for avoiding errors
		for x in range( len(indicesToOperateOn)):

			currentValues = listValues[x].split(',')
			arrayPosition = indicesToOperateOn[x]

			cmds.setAttr( '%s.%s[%s].%s' % ( node, mainAttr, arrayPosition, valAttr ), float(currentValues[0]))
			cmds.setAttr( '%s.%s[%s].%s' % ( node, mainAttr, arrayPosition, posAttr ), float(currentValues[1]))
			cmds.setAttr( '%s.%s[%s].%s' % ( node, mainAttr, arrayPosition, intAttr ), float(currentValues[2]))
			
			# print 'value to set', currentValues
			# print 'Set this point ID: ', x
	


	# =============================================================================================




def changeGradientControlNoAttrShape( widget, optionVarName, listValues):
	'''
	This method will use the current provided list of values in this format: [u'0.5,0,2', u'0.5,1,2', ..., ]
	It will change the shape of the gradient control accordingly
	'''
	#========== GRAD CONTROL NO ATTR
	#Very important TRICK, let the gradient control forget about its option
	cmds.gradientControlNoAttr( widget, e=True, optionVar='' )
	#Remove the current optionVar
	cmds.optionVar( rm = optionVarName )
	
	#Now for every value in the list, we build the optionVar
	for value in listValues:
		cmds.optionVar(stringValueAppend=[ optionVarName, value ] )
		
	#Re attach the option var to the control
	cmds.gradientControlNoAttr( widget, e=True, optionVar= optionVarName )
	
	
	
	