#!/usr/bin/env python
# -*- coding: utf-8 -*-


import maya.cmds as cmds



#Create a "new style class" inerithing from object
class HmOptionWindow(object):
	'''
	Thiss class implement the basic options for a Hard Mesh options window
	'''
	
	def __init__(self, windowName):
		#Here we initialize common variables
		if ' ' in windowName:
			cmds.error('name contains spaces! this would cause problems..')

		self.winName = windowName
		self.winTitle = windowName.replace ('_', ' ') #Remove underscores for name

		self.commandName = self.winTitle
		self.defauldDim = [600, 300]
		self.buttonHeight = 26
		self.btnSpace = 4
		self.menuItems = ['Edit', 'Help']
		self.executeTooltip = 'Sobstitute with useful indications'
		



		
	def insertContent(self):
		'''
		windowContent , this is an empty method to override in the child classes
		'''
		pass
	



	def buildWindow(self, *Args):
		'''
		This method build all the elements for the window, but still don't show it to the user.
		This because we can get the values from the options whitout the need to display it
		'''

		#Control if already exist
		if (cmds.window( self.winName, exists=True)):
			cmds.deleteUI( self.winName )


		#Create the window
		windowName = cmds.window(self.winName, title= self.winTitle, w = self.defauldDim[0], h = self.defauldDim[1], menuBar=True )
		
		#Create a script job, so every time window is closed, we save its size
		#we will create it...

		#Add the menu bar and the main scroll layout
		self.mainMenu()

		#Add an hidden tab and frame layout	
		#optionBoxTabLayout = cmds.tabLayout( tabsVisible = False, childResizable = True ) #scrollable = True, , 
		form = cmds.formLayout()
		
		mainScroll = cmds.scrollLayout(cr=True) #, bv=True ) # 
		cmds.frameLayout( label = '%s Options' % self.winTitle, collapsable= False, mh=10, mw=10 ) #bgs=True,
		
		
		
		# print 'CURRENT BEFORE CONTENT PARENT', cmds.setParent( q=True, m=True )
		#Add the content, this procedure now is empty. Will be overrided in the child classes.
		self.insertContent()		
		# print 'CURRENT PARENT', cmds.setParent( q=True, m=True )

		#Add the bottom main buttons
		self.buttonRow = cmds.rowLayout(adjustableColumn = 1, rowAttach = [1, 'bottom', 4 ] ) #we parent them directly to the window, so they will resize right
		self.mainButtons()
		
		cmds.formLayout(form, edit=True, 
		
		attachForm=[(mainScroll, 'top', 15), (mainScroll, 'left', 5), (mainScroll, 'right', 5), (self.buttonRow, 'bottom', 5), (self.buttonRow, 'right', 5), (self.buttonRow, 'left', 5) ],
		attachControl=[(mainScroll , 'bottom', 5, self.buttonRow )] 
		
		)


		#==============================================================	

		
	def buildAndShowWindow(self, *Args):
		'''
		This method builds and show the window to the user, used when the options are called
		'''
		self.buildWindow()
		cmds.showWindow( self.winName )

	def deleteWindow(self, *Args):
		'''
		Pretty easy
		'''
		cmds.deleteUI( self.winName )


	def mainMenu(self):
		'''
		This implements the main menu items
		'''
		self.menuBarLayout = cmds.menuBarLayout('mainLayout')
		cmds.menu( label='Edit' )
		cmds.menuItem( label='Save settings', c = self.saveOptions )
		cmds.menuItem( label='Reset settings', c = self.resetOptions)
		# cmds.menu( label='Help', helpMenu=True)
		# cmds.menuItem( label='Help on %s' % self.winName, c = self.openHelp )


	def mainButtons(self):
		'''
		This implements the main buttons arranged in a form layout
		'''
		#Add the main buttons
		buttonForm = cmds.formLayout()
		cmds.button('applyAndCloseBtn', label= self.commandName, recomputeSize = False, height = self.buttonHeight, c = self.performCmd, ann = self.executeTooltip )
		cmds.button('applyBtn', label = 'apply',  recomputeSize = False, height = self.buttonHeight, c = self.applyCmd )
		cmds.button('closeBtn', label = 'cancel',  recomputeSize = False, height = self.buttonHeight, c= self.cancelCmd )






		#Align the buttons
		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,

			attachForm =		[ 	
								('applyAndCloseBtn', 		'top',  	0 ),
								('applyAndCloseBtn', 		'left',  	self.btnSpace ),
								('applyAndCloseBtn', 		'bottom',  	0 ) 
								],
			attachPosition = 	[ 	
								('applyAndCloseBtn', 		'right',  	self.btnSpace,	33 )
								]
			)

		#==============================================================	

		
		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,
			attachForm =		[ 		
								('applyBtn', 				'top',  	0 ),
								('applyBtn', 				'bottom',  	0 )
								],
			attachPosition = 	[ 	
								('applyBtn', 				'left',  	self.btnSpace,		33 ),
							 	('applyBtn', 				'right',  	self.btnSpace,		67 )
							 	]	
			)


		#==============================================================	
		

		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,		 		
			attachForm =		[ 	
								('closeBtn', 				'top',  	0 ),
								('closeBtn', 				'bottom',  	0 ),
								('closeBtn', 				'right',  	self.btnSpace )
								],	
			attachPosition = 	[ 
								('closeBtn', 				'left',  	self.btnSpace,		67 )
							 	]			
			)
		#==============================================================	





	#======================================================================================
	def saveOptions(self, *Args):
		'''
		We use this method for saving the option between the various maya sessions
		'''
		print 'Options saved for %s' % self.winName

	def resetOptions(self, *Args):
		'''
		We use this method to reset the options tho the defined default values
		'''
		print 'Options reset for %s' % self.winName

	def openHelp(self, *Args):
		'''
		Here we will open the help sections related to the passed command
		'''
		print 'Will open the help of %s' % self.winName
	

	#======================================================================================

	def applyCmd(self, *Args):
		'''
		Execute command and leave window open
		'''
		# print 'command executed'
		pass


	def cancelCmd(self, *Args):
		'''
		do nothing and close the window
		'''
		self.deleteWindow()


	def performCmd(self, *Args):
		'''
		Perform command and close the window
		'''
		self.applyCmd()
		self.cancelCmd()

