#!/usr/bin/env python
# -*- coding: utf-8 -*-


######################################
# Questa classe implementa l interfaccia per lavorare con HardMesh

# Data: 04-02-2015
# Autore: Costantino Fracas
# Versione: 0.3


######################################

import os
import hmScripts as root
import maya.cmds as cmds
import xml.etree.ElementTree as ET


import hmScripts.UI.hmGlobalVariables as gVar
import hmScripts.UI.hmUiCommonUtils as uiUtils
import hmScripts.UI.hmOptionsHardMeshOperation as hardMeshOpOptions
import hmScripts.UI.hmOptionsIntersectionCurves as intersectionCurves
import hmScripts.Main.hmSelectionHandling as hmSelect


#Import the operation window
import hmScripts.UI.hmWindowHardMeshOperation as hardMeshOperationWindow


reload(hardMeshOperationWindow)



reload(gVar)
reload(uiUtils)
reload(hardMeshOpOptions)
reload(intersectionCurves)
reload(hmSelect)




'''
Per lo shader..
import maya.cmds as cmds

newNode = cmds.shadingNode( 'blinn', asShader=True )
newNodeAttr = newNode + '.normalCamera'
cmds.window()
cmds.columnLayout()
cmds.attrNavigationControlGrp( l='bump mapping', at=newNodeAttr )
cmds.showWindow()


'''

rootPath = root.__path__ [0]



class HardMeshDockUI():
	def __init__(self):
		#Query maya version
		self.mayaVersion = cmds.about( v=True) 
		
		#Global Frame Width
		self.frameWidth = 400
		
		
		#================================================
		#=========== Global Evaluation ==============
		#================================================
		self.computation = True
		#================================================
		
		#================================================
		#============ General Menu Options ==============
		#================================================
		self.showCurrentResult = True #This option shows in the viewport only the result for the selected compound 
		self.originalMeshWireframe = True
		self.usePreferredShader = True 
		self.autoSwitchTab = True
		
		#Siccome stiamo forzando le opzioni con dei valori di default fissi, niente option var, settiamo sempre lo shader di default
		cmds.optionVar(stringValue = [gVar.shaderOptionVarName, gVar.darkPhong] )
		
		
		#================================================
		#=============== Network Elements ===============
		#================================================
		#Store meshes
		# self.originalMesh = None
		self.currentMesh = None 
		
		#Store compounds
		# self.originalCompound = None #Here we save the first, original compound. This is the last in the chain
		self.lastCompound = None

		
		self.orderedCompoundsList = None
		self.currentCompoundsType = None
		self.compoundMenuItems = []

				
		#Store curves
		self.lastCurves = None
		self.currentCurves = None
		
		
		#Related nodes, compiled by: storeNetworkElements()
		self.currentOrderToSplit = None
		self.currentCurvesGrp = None 
		self.currentNetwork = None
		self.currentInputMeshes = None 
		#================================================
		
		
		
		
		#================================================
		#=========== Global Subdiv Options ==============
		#================================================
		self.currentSmoothAttributes = []
		#================================================
		
		
		
		
		
		
		#================================================
		#======= Windows and elements names =============
		#================================================
		self.windowName = 'Hard_Mesh_tools_Interactive'
		self.windowTitle = 'Hard Mesh'
		self.tabAName = 'Smart Edit'
		self.tabBName = 'Fast Create'
		self.origLayerName = gVar.origLayerName #Utility layer for hiding input mesh
		self.outLayerName = gVar.outLayerName #Utility layer for boolean meshes
		self.curvesLayerName = gVar.curvesLayerName #Utility layer for curves on meshes
		#================================================
		
		
		
		
		#================================================
		#============ Attributes Update =================
		#================================================
		#create a script job that will update the interface when a module is selected.		
		self.jobNum = cmds.scriptJob( e= ["SelectionChanged",self.checkAndUpdateAttributes] )
		self.jobNum1 = cmds.scriptJob( e= ["SceneOpened",self.updateLayersJob] )
		self.deleteJobNum = None


		#We use only the mesh related attributes
		self.origMeshIdentifyer = gVar.outMeshToCompound	
		self.inMeshFromCompoundIdentifyer = gVar.inMeshFromCompound
		

		#================================================
		
		
		
		#================================================
		#========== Ui Elemenst class Init ==============
		#================================================
		self.hmOpWin = hardMeshOperationWindow.HmHardMeshOperationWindow()

		
		self.hardMeshEditContent = hardMeshOpOptions.HardMeshOperationOptionsUI(asParameterChanger=True)
		self.hardMeshOpContent = hardMeshOpOptions.HardMeshOperationOptionsUI()
		self.intersectionCrvContent = intersectionCurves.IntersectionCurvesOptionsUI()
		

		#================================================

		
		
		#================================================
		#============ Tooltips Dictionary ===============
		#================================================
		self.tooltipArray = {
		'MeshSetupHmOffset' : 'Use the hmOffsetCurveNode',
		
		}
		
		#Links
		puxSite = 'http://www.pux-3d.com'
		tree = ET.parse('%s\\UI\documentation_link.xml' % rootPath)
		root = tree.getroot()
		self.currentDokLink = root.text
		
		#================================================
		
		

		
		#================================================
		#========== Interface elements build ============
		#================================================
		# Delete if exist
		if (cmds.dockControl( self.windowName, exists=True)):
			cmds.deleteUI( self.windowName )

		# ======= Create content

		#==== Window
		self.hmWin = cmds.window( menuBar=True )
		self.mainMenu()
		cmds.scrollLayout(cr=True) #Questo scroll si espande.. bene..
		
		#==== Form e Tabs
		#We turn off the fast create and use just one tab for now. We will keep this for future developments where the second tab may be used for other functionalities
		self.tabs = cmds.tabLayout(innerMarginWidth=5, innerMarginHeight=5, cc=self.updateLayers, tabsVisible = False) ###########################

		#==== Main tab
		tab01 = self.smartEdit( self.tabs )		
		
		#==== Sub tab
		tab02 = self.fastCreate( self.tabs )

		cmds.tabLayout( self.tabs, edit=True, tabLabel=((tab01, self.tabAName ), (tab02, self.tabBName ) ) )
		
		#===== Make window a dock control
		# dock = cmds.dockControl( self.windowName, label= self.windowTitle , area='right', allowedArea = ['left', 'right'], content= self.hmWin, closeCommand = self.killScriptJob )
		dock = cmds.dockControl( self.windowName, label= self.windowTitle , area='left', allowedArea = ['left', 'right'], content= self.hmWin )



		
		
		# if self.mayaVersion <= 2017:

		# 	# Delete if exist
		# 	if (cmds.dockControl( self.windowName, exists=True)):
		# 		cmds.deleteUI( self.windowName )

		# 	# ======= Create content

		# 	#==== Window
		# 	self.hmWin = cmds.window( menuBar=True )
		# 	self.mainMenu()
		# 	cmds.scrollLayout(cr=True) #Questo scroll si espande.. bene..
			
		# 	#==== Form e Tabs
		# 	#We turn off the fast create and use just one tab for now. We will keep this for future developments where the second tab may be used for other functionalities
		# 	self.tabs = cmds.tabLayout(innerMarginWidth=5, innerMarginHeight=5, cc=self.updateLayers, tabsVisible = False) ###########################

		# 	#==== Main tab
		# 	tab01 = self.smartEdit( self.tabs )		
			
		# 	#==== Sub tab
		# 	tab02 = self.fastCreate( self.tabs )

		# 	cmds.tabLayout( self.tabs, edit=True, tabLabel=((tab01, self.tabAName ), (tab02, self.tabBName ) ) )
			
		# 	#===== Make window a dock control
		# 	# dock = cmds.dockControl( self.windowName, label= self.windowTitle , area='right', allowedArea = ['left', 'right'], content= self.hmWin, closeCommand = self.killScriptJob )
		# 	dock = cmds.dockControl( self.windowName, label= self.windowTitle , area='left', allowedArea = ['left', 'right'], content= self.hmWin )





		# if self.mayaVersion >= 2017:


		# 	# Delete if exist
		# 	if (cmds.workspaceControl( self.windowName, exists=True)):
		# 		cmds.deleteUI( self.windowName )
			
			
		# 	#===== Make a new workspaceControl
		# 	cmds.workspaceControl(	self.windowName, 
		# 							label = self.windowTitle,
		# 							uiScript = self.newDockableWin,
		# 	                        dockToControl = ['MainPane', 'left'],
		# 	                        initialWidth = 200,
		# 	                        initialHeight = 400,
		# 	                        loadImmediately = True )

		# uiScript = 'import sys\ncurrent_module = sys.modules[__name__]\nprint current_module',	
		#====================== SCRIPT JOB KILLING =============================================
		#Create a script job that kills the first one once the window is closed... in this way we will avoid the close command
		cmds.scriptJob( runOnce=True, uiDeleted = [self.windowName, self.killScriptJob] )
	

	def newDockableWin(self):
		cmds.formLayout()
		cmds.button()	
		
	def mainMenu(self):
		menuBarLayout = cmds.menuBarLayout()
		
		#We will add these features after the 2.0 first release 
		cmds.menu( label='Utility', tearOff = True)
		# cmds.menuItem( label='Revert Base Mesh', ann = 'Remove selected meshes from layer and give it back the shader. No selection will do it for all the inputs.', c= self.revertMeshes )
		# cmds.menuItem( divider=True )
		self.delInMesh = cmds.menuItem( label='Delete Input Meshes', checkBox=True )
		cmds.menuItem( label='Clean Selected', ann = 'Delete all the related hm nodes from selected,\n and keep only the result mesh.', c=self.cleanSelected )
		cmds.menuItem( label='Clean Scene', ann = 'Delete all the related hm nodes and layers,\n and keep only the resulting meshes.', c=self.cleanScene )
		
		cmds.menuItem( divider=True )
		cmds.menuItem( label='Remove last operator', ann = 'Remove the last operator', c=self.removeLastCompound )
		# cmds.menuItem( label='Remove selected operator', ann = 'Remove the current operator', c=self.removeCurrentCompound, en=False)
		
		
		
		
		cmds.menu( label='Options', tearOff = True)
		
		#===============================================
		if self.mayaVersion <= 2014:
			cmds.menuItem(divider=True)
		else:
			cmds.menuItem(divider=True, dividerLabel = 'Editing')
		#===============================================	
		
		cmds.menuItem( label='Switch mesh to operator', ann = 'When the resulting mesh is made from differend operations, show only the mesh for the resulting operator', cb=True, c=self.toggleShowCurrentResult )
		cmds.menuItem( label='Wire Source Meshes', ann = 'Hide source meshes with a wireframe', cb=True, c = self.toggleWireframeMeshes )
		
		
		#===============================================
		if self.mayaVersion <= 2014:
			cmds.menuItem(divider=True)
		else:
			cmds.menuItem(divider=True, dividerLabel = 'Creation')
		#===============================================	

		cmds.menuItem( label='Use HM Shader', ann = 'Use a phong shader for the output meshes', cb=True, c = self.toggleShader )
		# cmds.menuItem( label='Auto Switch to Smart Edit', ann = 'Auto switch to the smart edit after creating a new hm operation', cb=True, c = self.toggleAutoSwitch )
		#We will use this for allowing the user to set a preferred shader...
		# cmds.menuItem( label='Set preferred shader', ann = 'Set a different shader than the default lambert for mesh output')
		# cmds.menuItem( label='Set preferred shader', ob=True )
		
		
		
		cmds.menu( label='Help and about', helpMenu=True, tearOff = True )
		cmds.menuItem( label='Help on Hard Mesh', c= self.openHtmlHelp )
		cmds.menuItem( label='About pux 3d', c= self.openHtmlAbout )
		cmds.menuItem( label='Version info', c= self.openInfoWin )
		
	
	
	def smartEdit(self, mainParent):
		layout = cmds.columnLayout(adjustableColumn = True, parent=mainParent )

		
		cmds.rowLayout( numberOfColumns=9 )
		cmds.shelfButton(image1="hm2_opUnion.png", ann = 'Create a new Union operation. double click for creation options', c = self.executeOperationAs1, dcc = self.openWindowsAs1 )
		cmds.shelfButton(image1="hm2_opDiffAB.png", ndp=True, ann = 'Create a new Difference A-B operation. double click for creation options', c = self.executeOperationAs2, dcc = self.openWindowsAs2 )
		cmds.shelfButton(image1="hm2_opDiffBA.png",  ndp=True, ann = 'Create a new Difference B-A operation. double click for creation options', c = self.executeOperationAs3, dcc = self.openWindowsAs3 )
		cmds.shelfButton(image1="hm2_opInter.png",  ndp=True, ann = 'Create a new Intersection operation. double click for creation options', c = self.executeOperationAs4, dcc = self.openWindowsAs4 )
		cmds.shelfButton(image1="hm2_opPanAB.png", ndp=True, ann = 'Create a new Panel A-B operation. double click for creation options', c = self.executeOperationAs5, dcc = self.openWindowsAs5 )
		cmds.shelfButton(image1="hm2_opPanBA.png",  ndp=True, ann = 'Create a new Panel B-A operation. double click for creation options', c = self.executeOperationAs6, dcc = self.openWindowsAs6 )
		cmds.separator(horizontal = True, w = 20, vis = False)
		# == Additional, future commands
		cmds.shelfButton(image1="polyBooleansUnion.png",  ndp=True, c = 'print 1', dcc = 'print 1', vis=False )
		cmds.shelfButton(image1="polyUnite.png", ndp=True, c = 'print 1', dcc = 'print 1', vis=False ) # ann = self.tooltipArray['Visibility'],
		#Set parent back to column
		cmds.setParent( '..' )
		
		cmds.separator( width = 30, height=10, style='in' )
		
		self.operatorsRow = cmds.rowLayout(numberOfColumns=7, rowAttach=[ 1, 'both', 5 ])
		self.compoundsOptionMenu = cmds.optionMenu( w=200, cc = self.changeCompoundFromMenu, noBackground= True  )
		self.compoundMenuItems.append( cmds.menuItem( label = 'Select A mesh...', parent = self.compoundsOptionMenu ) )
		cmds.separator(horizontal = True, w = 20, vis = False)
		# cmds.text( label='Global Sdiv' )
		cmds.shelfButton(image1='polySmooth.png',  imageOverlayLabel='More', w = 30, c=self.increaseGlobalSmooth,  ann = 'increase input meshes subdivisions')
		cmds.shelfButton(image1= 'polyReduce.png',  imageOverlayLabel='Less', w = 30, c=self.decreaseGlobalSmooth, ann = 'decrease input meshes subdivisions')
		cmds.separator(horizontal = True, w = 10, vis = False)
		cmds.checkBox( 'cbEvaluation', label = 'Compute', v = self.computation, cc = self.toggleComputation, noBackground= True  )
		
		#Set the parent to the mayn column layout, this avoids the "too many childre in row layour bla bla bla.."
		cmds.setParent( '..' )
		
		#==============================================================================================================================================================================
		cmds.separator( width = 30, height=20, style='in' )
		#==============================================================================================================================================================================	
		
		
		cmds.rowLayout(nc = 7 )
		# button = cmds.symbolButton(image= 'toggle_icon.png', c = self.toggleHm, ann = self.tooltipArray['Visibility'] ) #INSERT COMMAND HERE
		cmds.text( label='Layers:' )
		cmds.separator(horizontal = True, w = 30, vis = False)
		cmds.checkBox( 'cbBase', label = 'Base Mesh', v = self.getLayerStatus( self.origLayerName ), ofc = self.hideBaseLayer, onc = self.showBaseLayer)
		cmds.checkBox( 'cbResult', label = 'Result', v = self.getLayerStatus( self.outLayerName ), ofc = self.hideResultLayer, onc = self.showResultLayer)
		cmds.checkBox( 'cbCurve', label = 'Curves', v = self.getLayerStatus( self.curvesLayerName ), ofc = self.hideCurveLayer, onc = self.showCurveLayer)
		cmds.separator(horizontal = True, w = 30, vis = False)
		cmds.button(label = 'wireframe', c=self.hilightingToggle )
		
		#Set the parent to the mayn column layout, this avoids the "too many childre in row layour bla bla bla.."
		cmds.setParent( '..' )
			

		
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		
		
		#===== Hard Mesh Parameters
		# cmds.rowLayout(nc = 1, adj=1, columnAttach = [ 1, 'both', 3 ])
		self.optionsFrame = cmds.frameLayout( label = 'Basic Parameters', collapsable= False, collapse = False, enable = False, w= self.frameWidth )
		self.hardMeshEditContent.createUiElements()
		
		# #Set the parent, for the next controls, to the mayn column layout, this avoids the "too many children in row layour bla bla bla.."
		cmds.setParent( '..' )
		
		
		# #==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		# #==============================================================================================================================================================================	
		
		
		
		


		
		#===== Custom Channel Box
		# cmds.channelBox( 'hnChBox', h=400 )

		
		return layout

	
	def fastCreate(self, mainParent):

		layout = cmds.columnLayout( adjustableColumn = True, parent=mainParent )
		
		'''
		#===== Hard Mesh Operation
		cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ], adj = 2 )
		button = cmds.symbolButton(image= 'meshSolver_icon.png', c = self.performHmOperation ) #INSERT COMMAND HERE
		#================== Options
		cmds.frameLayout( label = 'New Hard Mesh Operation', collapsable= True, collapse = True, w= self.frameWidth )
		self.hardMeshOpContent.createUiElements()
		
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		
		#===== Intersection Curves
		cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ], adj = 2  )
		button = cmds.symbolButton(image= 'meshSetup_icon.png', c = self.intersectionCrvContent.performCommand ) #INSERT COMMAND HERE
		#================== Options
		cmds.frameLayout( label = 'Intersection Curves', collapsable= True, collapse = True, w= self.frameWidth )
		self.intersectionCrvContent.createUiElements()
		
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		
		# ===== Split Command
		cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ], adj = 2  )
		button = cmds.symbolButton(image= 'splitWithCurve_icon.png', c = self.splitContent.performCommand ) #INSERT COMMAND HERE
		
		#================== Options
		cmds.frameLayout( label = 'Split with curve', collapsable= True, collapse = True, w= self.frameWidth  )
		self.splitContent.createUiElements()	
		
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		
		
		
		#===== Blend Setup
		cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ], adj = 2  )
		button = cmds.symbolButton(image= 'blendMesh_icon.png', c = self.blendContent.performCommand ) #INSERT COMMAND HERE
		
		#================== Options
		cmds.frameLayout( label = 'Blend from edges', collapsable= True, collapse = True, w= self.frameWidth  )
		self.blendContent.createUiElements()
	
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		
		#===== Snap Command
		cmds.rowLayout(nc = 2, rowAttach =[ (1, 'top', 5), (2, 'top', 5) ], adj = 2  )
		button = cmds.symbolButton(image= 'snapVertex_icon.png' , c = self.snapContent.performCommand ) #INSERT COMMAND HERE
		
		#================== Options
		cmds.frameLayout( label = 'Snap to curve', collapsable= True, collapse = True, w= self.frameWidth  )
		self.snapContent.createUiElements()	
						
		#==============================================================================================================================================================================	
		cmds.separator( height = 20, style='in' )
		#==============================================================================================================================================================================	
		
		'''
		return layout

	#================================================
	#==============      COMMANDS      ==============
	#================================================	
	
	# ==== Operations windows parameters forcing...
	# ==== We create different methods to call, one for the execute and one for the options forcing in both the operation.
	def openWindowsAs1(self, *Args):
			self.openWindowsAs(1)
	def openWindowsAs2(self, *Args):
			self.openWindowsAs(2)			
	def openWindowsAs3(self, *Args):
			self.openWindowsAs(3)			
	def openWindowsAs4(self, *Args):
			self.openWindowsAs(4)
	def openWindowsAs5(self, *Args):
			self.openWindowsAs(5)
	def openWindowsAs6(self, *Args):
			self.openWindowsAs(6)


	def executeOperationAs1(self, *Args):
		self.executeOperationAs(1)
	def executeOperationAs2(self, *Args):
		self.executeOperationAs(2)
	def executeOperationAs3(self, *Args):
		self.executeOperationAs(3)
	def executeOperationAs4(self, *Args):
		self.executeOperationAs(4)
	def executeOperationAs5(self, *Args):
		self.executeOperationAs(5)
	def executeOperationAs6(self, *Args):
		self.executeOperationAs(6)
			
	# === This two are the ones that actually set the options through the instance class		
	def openWindowsAs(self, val ):
		self.hmOpWin.setForcedAttributes(val)
		self.hmOpWin.buildAndShowWindow()
		
		
	def executeOperationAs(self, val):
		self.hmOpWin.setForcedAttributes(val)
		self.hmOpWin.performFromMenu()
		
	

	
	def toggleComputation(self, *Args):
		'''
		This command turns on and off all the hmCompound nodes in scene
		'''
		#Get all compound nodes
		compounds = cmds.ls(et='hmCompoundNode')
		offsets = cmds.ls(et='hmOffsetCurveNode')
		

		
		
		if self.computation == 1:
			#Turn it off
			for compound in compounds:
				#print 'turn off for:', compound
				cmds.setAttr('%s.compute' % compound, 0)
			for offset in offsets:
				cmds.setAttr('%s.compute' % offset, 0)
			# Set the var value
			self.computation = 0
			
			#Set the background color of the toggle to red for letting know the user that no evaluations are occurring..
			cmds.rowLayout( self.operatorsRow, e=True, enableBackground = True, backgroundColor = [255,0,0] )
			
			
		elif self.computation == 0:
			#Turn it on
			for compound in compounds:
				#print 'turn on for:', compound
				cmds.setAttr('%s.compute' % compound, 1)	
			for offset in offsets:
				cmds.setAttr('%s.compute' % offset, 1)		
			# Set the var value
			self.computation = 1
			
			# Set the back color back to neutral
			cmds.rowLayout( self.operatorsRow, e=True, noBackground= True )
			cmds.optionMenu(self.compoundsOptionMenu, e=True, noBackground= True  )
			
			
	def toggleHm( self, *Args ):
		baseVis = 1
		boleVis = 1
		curveVis = 1
		#Prendiamo lo stato attuale
		if self.layerExist( self.origLayerName ):
			baseVis = cmds.getAttr( '%s.visibility' % self.origLayerName )
			
		if self.layerExist( self.outLayerName ):
			boleVis = cmds.getAttr( '%s.visibility' % self.outLayerName )
			
		if self.layerExist( self.curvesLayerName ):
			curveVis = cmds.getAttr( '%s.visibility' % self.curvesLayerName )
		
		#Settiamo gli stati
		if baseVis == 0 and boleVis == 0 and curveVis == 0:
			#Layers
			self.showBaseLayer()
			self.showResultLayer()
			self.showCurveLayer()
			
			#Ui
			cmds.checkBox( 'cbBase', edit=True, v=1 )
			cmds.checkBox( 'cbResult', edit=True, v=1 )
			cmds.checkBox( 'cbCurve', edit=True, v=1 )
			
		if baseVis == 1 and boleVis == 1 and curveVis == 1:
			#Layers
			self.showBaseLayer()
			self.hideResultLayer()
			self.hideCurveLayer()
			
			#Ui
			cmds.checkBox( 'cbBase', edit=True, v=1 )
			cmds.checkBox( 'cbResult', edit=True, v=0 )
			cmds.checkBox( 'cbCurve', edit=True, v=0 )
			
		elif baseVis == 1 and boleVis == 0 and curveVis == 0:
			#Layers
			self.hideBaseLayer()
			self.showResultLayer()
			
			#Ui
			cmds.checkBox( 'cbBase', edit=True, v=0 )
			cmds.checkBox( 'cbResult', edit=True, v=1 )		
			
		elif baseVis == 0 and boleVis == 1 and curveVis == 0:
			#Layers
			self.hideResultLayer()
			self.showCurveLayer()
			
			#Ui
			cmds.checkBox( 'cbResult', edit=True, v=0 )
			cmds.checkBox( 'cbCurve', edit=True, v=1 )
			
		elif baseVis == 0 and boleVis == 0 and curveVis == 1:
			#Layers
			self.hideCurveLayer()
			
			#Ui
			cmds.checkBox( 'cbCurve', edit=True, v=0 )
		else:
			#Layers
			self.showBaseLayer()
			self.showResultLayer()
			self.showCurveLayer()
			
			#Ui
			cmds.checkBox( 'cbBase', edit=True, v=1 )
			cmds.checkBox( 'cbResult', edit=True, v=1 )
			cmds.checkBox( 'cbCurve', edit=True, v=1 )
			

	def increaseGlobalSmooth(self, *Args):
		self.changeGlobalSmooth(+1)
	def decreaseGlobalSmooth(self, *Args):
		self.changeGlobalSmooth(-1)
		
	def changeGlobalSmooth(self, value):
		'''
		This mothod changes the global smooth of the current network
		'''
					

		if len(self.currentSmoothAttributes) > 0:	
			# print 'I have these attributes:', self.currentSmoothAttributes	
			for attribute in self.currentSmoothAttributes:
				#Get the current value
				currSmooth = cmds.getAttr( attribute )

				# print 'this is the current value:', currSmooth
				
				if value > 0:
					newSmooth = uiUtils.clamp( currSmooth + value, 0, 5 )
				if value < 0:
					newSmooth = uiUtils.clamp( currSmooth - abs(value), 0, 5 )	
					#print 'current smooth: ', currSmooth, 'devo rimuovere', value, 'setto ', newSmooth
					
				#Set the attribute..
				cmds.setAttr( attribute, newSmooth)
		
		#Update the UI
		self.hardMeshEditContent.updateSmoothUI()


	#================================================
	#===============      LAYERS     ================
	#================================================			

	def showBaseLayer( self, *Args):
		if self.layerExist( self.origLayerName ):
			cmds.setAttr( '%s.visibility' % self.origLayerName, 1 )
			#Set the result layer in reference
			cmds.setAttr( '%s.displayType' % self.outLayerName, 2 )

	def hideBaseLayer( self, *Args):
		if self.layerExist( self.origLayerName ):
			cmds.setAttr( '%s.visibility' % self.origLayerName, 0 )
			#Set the result layer selectable
			cmds.setAttr( '%s.displayType' % self.outLayerName, 0 )
			
	def showResultLayer( self, *Args):
		if self.layerExist( self.outLayerName ):
			cmds.setAttr( '%s.visibility' % self.outLayerName, 1 )

	def hideResultLayer( self, *Args):
		if self.layerExist( self.outLayerName ):
			cmds.setAttr( '%s.visibility' % self.outLayerName, 0 )
			
	def showCurveLayer( self, *Args):
		if self.layerExist( self.curvesLayerName ):
			cmds.setAttr( '%s.visibility' % self.curvesLayerName, 1 )

	def hideCurveLayer( self, *Args):
		if self.layerExist( self.curvesLayerName ):
			cmds.setAttr( '%s.visibility' % self.curvesLayerName, 0 )
	
	def hilightingToggle(self, *Args):
		'''
		Turns on and off the selection hilighting for the current persp view
		'''
		perspPanel = cmds.getPanel( withLabel='Persp View')
		if (cmds.modelEditor( perspPanel, query=1, sel=1)) == 1:
			cmds.modelEditor( perspPanel, edit=True, sel=0)
		else:
			cmds.modelEditor( perspPanel, edit=True, sel=1)
			
			
	#================================================
	#============      MENU COMMANDS     ============
	#================================================
	def removeLastCompound(self, *Args):
		'''
		This method deletes the related elements for the current compound in list
		'''
		# Save the curent selection
		selection = hmSelect.HmSelectionHandling()
		#Set custom messages
		selection.setErrorMessageIfZero('Please select 1 mesh generated from an Hard Mesh Operation.')
		selection.setErrorIfSmallThanMin('Please select 1 mesh generated from an Hard Mesh Operation.')
		selection.setErrorMessageIfOutLimit('Please select only 1 mesh.')
		sel = selection.validate(1,1,'mesh')

		# If we have not selected a resulting mesh, print an error because we want
		# to remove the last compound from a resulting mesh. removing it from
		# an input mesh can give errors.
		shape = cmds.listRelatives(sel[0], shapes=True)[0]
		if not cmds.listConnections(shape + '.inMesh', type = 'hmCompoundNode'):
			cmds.error('For removing the last operation we need a mesh generated from an Hard Mesh Operation.')
		

		#Store the relevant element of the current compound
		self.storeNetworkElements()


		#Id of the current compound in list...
		id = self.getCurrentCompoundId()
		
		# == If we have more than 1 compound in list
		if id > 0:
			previewCompound = self.orderedCompoundsList[id-1]

			# OLD TRANSFORM METHOD
			# connectedOutMesh = cmds.listConnections('%s.%s' % (previewCompound, gVar.outCompoundToMesh), d=True, type = 'transform') [0]

			# NEW SHAPE METHOD (returns the transform anyway)
			connectedOutMesh = cmds.listConnections('%s.%s' % (previewCompound, gVar.outCompoundToMesh), d=True, type = 'mesh') [0]


			# == Show the preview mesh
			cmds.setAttr('%s.visibility' % connectedOutMesh , 1)
			# == Remove the current result mesh from the input list
			print self.currentInputMeshes, 'ho questo'
			self.currentInputMeshes.remove(connectedOutMesh)
			print self.currentInputMeshes, 'ho questo'
			# Remove the smooth out
			self.cleanMeshSmoothOut([connectedOutMesh])

		

		# Print some messages
		# print 'current inp meshes:', self.currentInputMeshes
		# print 'devo cancellare: ', self.currentMesh, self.currentOrderToSplit, self.orderedCompoundsList[id], self.currentNetwork
			

		# === Delete the unwanted elements of current compound
		cmds.delete( self.currentMesh, self.currentOrderToSplit, self.orderedCompoundsList[id], self.currentNetwork )
		

		# == Revert the input meshes
		cmds.select(self.currentInputMeshes, r=True)
		self.revertMeshes()
		self.cleanMeshSmoothOut(self.currentInputMeshes)
		cmds.select(cl=True)
		
		'''
		cmds.error('in the end the networks connections are too complicated, you have to get also the intersectors, the conditions nodes ecc...maybe an asset would be ideal?')
		# ============ Compound is in the middle of the chain case =================
		
		
		# == We delete the input mesh num 1
		inMeshToDel = self.currentInputMeshes[1]
		inMeshToReconnect = self.currentInputMeshes[0]
		
		# connect prev compound to next compound, this is used for keeping track of connections from the UI
		cmds.connectAttr('%s.%s' % (prevCompound, gVar.outCompoundToMesh), '%s.%s' % (nextCompound, gVar.outCompoundToMesh), f=True )
		
		
		#Get the shapes of the mesh to reconnect
		reconnectShapes = cmds.listRelatives(inMeshToReconnect, s=True)
		#For the connections that we are doing we need the second shape, that came out from the smooth.
		outSmoothedShape = reconnectShapes[1]
		print outSmoothedShape, 'outSmoothedShape'
		
		#Get the order to split of the next compound
		nextOrderToSplit = cmds.listConnections('%s.inCurves' % nextCompound, s=True, d=False, type = 'hmOrderToSplitNode') [0]
		
		# Connect the mesh to reconnect to the order to split
		cmds.connectAttr('%s.outMesh' % outSmoothedShape, '%s.%s[%s]' % ( nextOrderToSplit, gVar.inMeshes, 0), f=True )		
		cmds.connectAttr('%s.worldMatrix[0]' % inMeshToReconnect, '%s.%s[%s]' % ( nextOrderToSplit, gVar.inMeshesMatrix, 0), f=True )
		
		# And to related compound
		cmds.connectAttr('%s.%s' %(inMeshToReconnect, gVar.outMeshToCompound), '%s.%s' %(nextCompound, gVar.inMeshForCompoundA), f=True)
		
		
		#Delete the unwanted elements of current compound
		cmds.delete(inMeshToDel, self.currentMesh, self.currentOrderToSplit, self.orderedCompoundsList[id], self.currentNetwork)
		
		
		cmds.error('in the end the networks connections are too complicated, you have to get also the intersectors, the conditions nodes ecc...maybe an asset would be ideal?')
		'''

	def removeCurrentCompound(self, *Args):
		'''
		NOT USED
		NOT USED
		NOT USED
		NOT USED
		This method deletes the related elements for the current compound in list
		'''
		#Store the relevant element of the current compound
		self.storeNetworkElements()
		#Id of the current compound in list...
		id = self.getCurrentCompoundId()
		#Get the preview and next compounds
		nextCompound = self.getCompoundToCompoundConnection(self.orderedCompoundsList[id], 'forward') [0]
		prevCompound = self.getCompoundToCompoundConnection(self.orderedCompoundsList[id], 'backward') [0]
		
		#Print some messages
		# print 'next:', nextCompound
		# print 'prev:', prevCompound
		# print 'devo cancellare: ', self.orderedCompoundsList[id]
		# print 'l outMesh corrente è:', self.currentMesh
		# print 'current inp meshes:', self.currentInputMeshes

		
		# ============ Compound is in the middle of the chain case =================
		
		
		# == We delete the input mesh num 1
		inMeshToDel = self.currentInputMeshes[1]
		inMeshToReconnect = self.currentInputMeshes[0]
		
		# connect prev compound to next compound, this is used for keeping track of connections from the UI
		cmds.connectAttr('%s.%s' % (prevCompound, gVar.outCompoundToMesh), '%s.%s' % (nextCompound, gVar.outCompoundToMesh), f=True )
		
		
		#Get the shapes of the mesh to reconnect
		reconnectShapes = cmds.listRelatives(inMeshToReconnect, s=True)
		#For the connections that we are doing we need the second shape, that came out from the smooth.
		outSmoothedShape = reconnectShapes[1]
		# print outSmoothedShape, 'outSmoothedShape'
		
		#Get the order to split of the next compound
		nextOrderToSplit = cmds.listConnections('%s.inCurves' % nextCompound, s=True, d=False, type = 'hmOrderToSplitNode') [0]
		
		# Connect the mesh to reconnect to the order to split
		cmds.connectAttr('%s.outMesh' % outSmoothedShape, '%s.%s[%s]' % ( nextOrderToSplit, gVar.inMeshes, 0), f=True )		
		cmds.connectAttr('%s.worldMatrix[0]' % inMeshToReconnect, '%s.%s[%s]' % ( nextOrderToSplit, gVar.inMeshesMatrix, 0), f=True )
		
		# And to related compound
		cmds.connectAttr('%s.%s' %(inMeshToReconnect, gVar.outMeshToCompound), '%s.%s' %(nextCompound, gVar.inMeshForCompoundA), f=True)
		
		
		#Delete the unwanted elements of current compound
		cmds.delete(inMeshToDel, self.currentMesh, self.currentOrderToSplit, self.orderedCompoundsList[id], self.currentNetwork)
		
		
		cmds.error('in the end the networks connections are too complicated, you have to get also the intersectors, the conditions nodes ecc...maybe an asset would be ideal?')
		

		
	def cleanMeshSmoothOut(self, inputList):
		'''
		This procedure deletes the output poly smooth node connected
		in the input meshes.
		'''
		# Cycle throught all passed meshes
		for inMesh in inputList:
			# print 'CONTROLLO', inMesh
			meshShapes = cmds.listRelatives(inMesh, shapes=True, fullPath=True)

			# Get the shape of the surrent mesh transform
			if meshShapes:
				# print meshShapes
				# Cycle through all the find shapes
				for shape in meshShapes:
					# print 'LE SHAPES SONO', shape
					# Get the soutput smooth node
					smoothNode = cmds.listConnections(shape, d=True, t='polySmoothFace')

					# If we find a smooth node, proceed and search an out mesh for it
					if smoothNode:
						# print 'LO SMOOTH E', smoothNode
						# Get the output of this smooth node
						intermediateMesh =  cmds.listConnections(smoothNode[0]+'.output', d=True, shapes=True, type='mesh')

						# If we find an out mesh for the smooth, we just need to delete it for having also the smooth eliminated
						if intermediateMesh:
							# print 'CANCELLO: ', intermediateMesh
							cmds.delete(intermediateMesh)
							break
						# Otherwhise just delete the smooth
						else:
							cmds.delete(smoothNode)
							break

				# After removing the ous stuff, remove also the attribute used for the queries
				attr = cmds.listAttr(inMesh, ud=True)
				# If we fint attributes to check
				if attr:
					if gVar.outMeshToCompound in attr:
						cmds.deleteAttr(inMesh + '.' + gVar.outMeshToCompound)

				# We should have now the mesh cleaned

		
	def revertMeshes(self, *Args):
		'''
		This method revert the selected mesh to original, removing the overrides and the layed assignment.
		'''

		#Get the objects in the layer
		if self.layerExist(self.origLayerName):
			objectsInLayer = cmds.editDisplayLayerMembers(self.origLayerName, q=True, fullNames=True)
		else:
			cmds.error('The scene does\'t have the layer %s, operation aborted.' % self.origLayerName)
		
		
		sel = cmds.ls(sl=True, long=True)
		
		# if something selected, query that belongs to the orig mesh layers
		if len(sel) != 0:
			for object in sel:
				#If the object is not an input mesh, return an error.
				if object not in objectsInLayer:
					cmds.error('The object %s is not an input mesh. Please select one input mesh or select nothing to revert all.' % object)
				else:
					#If it is an input mesh, remove from the layer
					# print 'object to remove from layer', object
					cmds.editDisplayLayerMembers('defaultLayer', object, noRecurse=True)
					#And activate the shading
					uiUtils.makeItWireframe(object, operation = 0)
					##Remove custom attributes and output smooth node..
					uiUtils.removeCustomAttr(object)
		else:			
			#Remove the objects from layer
			cmds.editDisplayLayerMembers('defaultLayer', objectsInLayer, noRecurse=True)
			

			# Remove out nodes
			# self.cleanMeshSmoothOut(objectsInLayer)		

			for object in objectsInLayer:			
					#And activate the shading
					uiUtils.makeItWireframe(object, operation = 0)
					#Remove custom attributes and output smooth node..
					uiUtils.removeCustomAttr(object)


	
	
	def cleanSelected(self, *Args):
		'''
		This method delete all the hardMesh related nodes from selected mesh. It finds all the connections indipendently if is selected an input or output mesh.
		'''
		sel = cmds.ls(sl=True, long=True)
		
		# In this list we store the found hmCompounds, hmOrderToSplit and hmNetwork found from curve
		deleteList = []
		inMeshesList = []
		
		#===== If one or more object selected...
		if len(sel) != 0:
			# Update the layers
			self.updateLayers()

			
			# ==== For every object in selection....
			for currentObj in sel:
			
				cmds.select(currentObj, r=True)
				# ===== Check that it have relevance. We just check the current self.orderedCompoundsList var since we already 
				# ===== check if the selection is relevent in the checkAndUpdateAttributes method.
				if self.orderedCompoundsList != None:
				
					#=========== If yes, cycle on all the related compounds found in the list. We already found that list in the main proc..
					#=========== For every compound 
					for compound in self.orderedCompoundsList:
						# #Get the order to split node...
						# orderToSplit = cmds.listConnections('%s.inCurves' % compound, s=True, d=False)[0] #, type = 'hmCompoundNode') 
						# #From compound get one input curve..
						# curve = cmds.listConnections('%s.inCurves' % orderToSplit, s=True, d=False)[0] #, type = 'hmCompoundNode')
						# #From curve get the related group, and store it for delete 
						# curveGrp = cmds.listRelatives(curve, p=True, f=True)[0]
						# currentNetwork = cmds.listRelatives(curveGrp, p=True, f=True)[0]
						# #Get the input meshes from the order to split connections...
						# inMeshes = cmds.listConnections('%s.inMeshes' % orderToSplit, s=True, d=False)
						
						# #Store the things to delete in the list
						# deleteList.extend( [compound, orderToSplit, currentNetwork ] )
						# inMeshesList.extend( inMeshes )

						# #Store the things to delete in the list
						self.storeNetworkElements( )
						deleteList.extend( [compound, self.currentOrderToSplit, self.currentNetwork ] )
						inMeshesList.extend( self.currentInputMeshes )


				# ==== If the selected object is not related to hard mesh...
				else:			
					#If not prompt an error 
					cmds.error('The selected object %s doesn\'t appear to be something related to HardMesh.' % sel)
					
					
				#====== if we are here we can proceed to delete stuff..	
				#Delete out meshes history
				
				#Dedlete unwanted nodes...		
				cmds.delete(deleteList)

				# Del in meshes if requires
				delInputs = cmds.menuItem(self.delInMesh, q=True, checkBox = True )

				if delInputs:
					cmds.delete(inMeshesList)
				else:
					cmds.select(self.currentInputMeshes)
					self.revertMeshes()
					self.cleanMeshSmoothOut(self.currentInputMeshes)	
				
		else:
			cmds.error('please select one or more hard mesh related objects: an input mesh or an output mesh.')
			
			
	def cleanScene(self, *Args):
		'''
		This method deletes all the HardMesh nodes, hierarchy and layers keeping just the output meshes.
		'''
		
		#Delete all HardMeshRelatedStuff..
		if self.layerExist( self.outLayerName ):
			#Get all the output meshes
			outMeshes = cmds.editDisplayLayerMembers(self.outLayerName, q=True, fullNames=True)		
			
			#Get all compound nodes
			compounds = cmds.ls(et='hmCompoundNode')
			orderToSplit =  cmds.ls(et='hmOrderToSplitNode')
			offsetNodes =  cmds.ls(et='hmOffsetCurveNode')
			
			#Delete their history
			cmds.delete(outMeshes, ch=True)
			
			#Delete hmNodes...
			cmds.delete(compounds, orderToSplit)
			
			#Get the curves
			curves = cmds.editDisplayLayerMembers(self.curvesLayerName, q=True, fullNames=True)		
			
			#Get the main hierarchy from curves
			currentNetwork = cmds.listRelatives(curves, p=True, f=True)
			
			#Delete it
			cmds.delete(currentNetwork)
			
			#Delete all the elements contained in the input mesh if required.
			delInputs = cmds.menuItem(self.delInMesh, q=True, checkBox = True )
			inMeshes = cmds.editDisplayLayerMembers(self.origLayerName, q=True, fullNames=True)	
			
			if delInputs:
				#Delete them 
				cmds.delete(inMeshes)	
			else:
				self.revertMeshes()
				self.cleanMeshSmoothOut(inMeshes)

			
			
		#If we arrive here without errors everything should be ok and we can delete all the HardMesh layers
		for layer in [self.outLayerName, self.curvesLayerName, self.origLayerName]:
			#If layer is found without problems...
			if self.layerExist(layer):
				cmds.delete( layer )
			else:
				cmds.warning('unable to find %s in scene, the name has changed or the layer have been deleted' % layer)
		
		
	def toggleShowCurrentResult(self, *Args):
		'''
		Switch between this option that shows in the viewport only the result for the selected compound 
		'''
		if self.showCurrentResult == True:
			self.showCurrentResult = False
		elif self.showCurrentResult == False:
			self.showCurrentResult = True

			
	def toggleWireframeMeshes(self, *Args):
		'''
		This method switch between wireframe and full shaded meshes
		'''
		#Get the list of original meshes	
		objectsInLayer = cmds.editDisplayLayerMembers(self.origLayerName, q=True, fullNames=True)
		
		if self.originalMeshWireframe == True:
			# print 'turnitoff'
			#Set the override off
			self.originalMeshWireframe = False
			for object in objectsInLayer:
				uiUtils.makeItWireframe(object, operation = 0)
				
		elif self.originalMeshWireframe == False:
			# print 'turniton'
			#Set the override on
			self.originalMeshWireframe = True
			for object in objectsInLayer:
				uiUtils.makeItWireframe(object, operation = 1)
	
		
	def toggleShader(self, *Args):
		'''
		This method activate and deactivate the preferred shader assignment
		'''
		if self.usePreferredShader == True:
	
			#If we don't want to use the preferred shader, delete the var.
			cmds.optionVar( remove = gVar.shaderOptionVarName )
			
			#Update the variable
			self.usePreferredShader = False
			
			# print 'optionVar Removed '
			
		elif self.usePreferredShader == False:
			#Prendiamo il preferred shader settato
			
			#O in questo caso forziamogli sempre lo stesso...
			cmds.optionVar(stringValue = [gVar.shaderOptionVarName, gVar.darkPhong] )
			
			# print 'optionVar con valore ', cmds.optionVar(q=gVar.shaderOptionVarName)
			#Update the variable
			self.usePreferredShader = True
		
	def toggleAutoSwitch(self, *Args):
		'''
		This method activate and deactivate the auto switch after an hm operation
		'''
		
		if self.autoSwitchTab  == True:
			#Update the variable
			self.autoSwitchTab  = False
			
		elif self.autoSwitchTab  == False:
			#Update the variable
			self.autoSwitchTab  = True


	def performHmOperation(self, *Args):
		'''
		Create a new hm operation and auto switch to the edit tab, if the option is on
		'''


		
		#Call command from class
		self.hardMeshOpContent.performCommand()
		#Auto switch checking 
		if self.autoSwitchTab == True:
			cmds.tabLayout( self.tabs, edit=True, selectTabIndex = 1 )
			

		
			
	def performHmSnap( self, *Args ):
		#Get Ui data
		delCrvVal = cmds.checkBox( 'cbSnapDelCrv', query=True, v=True)
		cmds.hmSnapVertexOnCurveCfg( dc= delCrvVal, cb = 0 )
		cmds.hmSnapVertexOnCurve()
	
	
	def openHtmlHelp(self, *Args ):
		os.startfile( self.currentDokLink )
		
	def openHtmlAbout( *Args ):
		os.startfile('http://www.pux-3d.com')

	def openInfoWin( *Args ):
		'''
		display info on plugin
		'''
		winName = 'infoWin'
		version = cmds.pluginInfo('hmTools', query=True, v=True )
		
		if (cmds.window( winName, exists=True)):
			cmds.deleteUI( winName )
		
		window = cmds.window( winName, title='Hard Mesh info',  widthHeight=(280, 135), sizeable=False )
		cmds.columnLayout( adjustableColumn=True )
		cmds.separator( width = 30, height=20, style='in' )
		cmds.text( label='Hard Mesh version %s\n\nPUX CORPORATE SA\n\nLigornetto - Switzerland\n' % version, align='center' )
		cmds.separator( width = 30, height=20, style='in' )
		cmds.setParent( '..' )
		cmds.showWindow( window )
		
	#================================================
	#============        UTILITY         ============
	#================================================
	
	def storeNetworkElements(self):
		'''
		We get the related elements of the current compound.
		We mainly use this for the delete of the current compound

		this method fills

		self.currentOrderToSplit with the current offset/order to split node.. ()
		self.currentCurvesGrp with the group of current out curves
		self.currentNetwork the name of the current network group
		self.currentInputMeshes the current input meshes

		'''
		id = self.getCurrentCompoundId()
		
		compound = self.orderedCompoundsList[id]
		
		#Get the order to split node...
		self.currentOrderToSplit = cmds.listConnections('%s.inCurves' % compound, s=True, d=False)[0] #, type = 'hmCompoundNode') 
		#From compound get one input curve..
		curve = cmds.listConnections('%s.inCurves' % self.currentOrderToSplit, s=True, d=False)[0] #, type = 'hmCompoundNode')
		#From curve get the related group, and store it for delete 
		self.currentCurvesGrp = cmds.listRelatives(curve, p=True, f=True)[0]
		self.currentNetwork = cmds.listRelatives(self.currentCurvesGrp, p=True, f=True)[0]
		# Now we get the poly smooth nodes
		polySmooth = cmds.listConnections('%s.inMeshes' % self.currentOrderToSplit, s=True, d=False)

		self.currentInputMeshes = []

		#Get the input meshes from the polySmooth connections...
		for smooth in polySmooth:
			inMesh = cmds.listConnections('%s.inputPolymesh' % smooth, s=True, d=False)[0]
			self.currentInputMeshes.append(inMesh)

		# print 'current input meshes are:', self.currentInputMeshes

						
	def storeCurrentOutMesh(self, *Args):
		'''
		Get the out mesh of currently selected operator..
		'''
		#Update ui, and set the current compound node as the editable for the options
		id = self.getCurrentCompoundId()
		
		
		# Since we are using the shape now, we skip this
		#Get the mesh related to the CURRENT selected compound
		# connectedMesh = cmds.listConnections('%s.%s' % (self.orderedCompoundsList[id], gVar.outCompoundToMesh), d=True, type = 'transform') 

		# We query the mesh types instead of the tranform types
		connectedMesh = cmds.listConnections('%s.%s' % (self.orderedCompoundsList[id], gVar.outCompoundToMesh), d=True, type = 'mesh') 
		
		if connectedMesh != None:
			self.currentMesh = connectedMesh[0]
		else:
			cmds.warning('unable to find connected mesh to: %s. It\'s probably a dead compound' % self.orderedCompoundsList[id] )	
			
			
			
	def killScriptJob(self, *Args):
		'''
		We call this command each time we close the window
		'''
		#Kill the scriptJob
		cmds.scriptJob( kill=self.jobNum, force=True)
		#Remove possible curves color override
		self.restoreCurvesColor()
		
		# print 'script job killed'
		
	def updateLayersJob(self, *Args):
		'''we call this method for having always up to date the layer status also when opening scene'''
		self.updateLayers()
			
	def checkAndUpdateAttributes(self, *Args):
		'''
		This method will check if the current selection is an hardmesh relevant mesh, and if yes it will refresh the attributes UI
		Otherwhise it will just pass.
		If the current window is closed (aka invisible), the job will be killed
		'''
		# ========= WARNING =============
		# pass
		# ===== Open a new undo chunk
		cmds.undoInfo(openChunk=True)
		
		
		# ========= WARNING =============

		if (cmds.dockControl( self.windowName, query=True, visible=True)) is True:
			#If the current selection contains a hm related mesh (checking if contains some hm attributes string)
			sel = cmds.ls(sl=True, long=True)
			
			if len(sel) > 0:
				
				# print 'Selection Script job %s executed' % self.jobNum
				#First of all, Clean the compound list and related ordetToSplit
				self.orderedCompoundsList = None
				self.currentCompoundsType = None
				
				#Get the first element of selection
				currentSelection =  sel[0]		
				
				#Now check it the selection has one of ours custom attributes
				queryResult = uiUtils.queryMeshType( currentSelection )
				hasAttribute = queryResult[0]
				
				# print 'queryResult:', queryResult
				
				#================================================
				#================ Mesh Has Relevance ============
				#================================================
				# Check if the current selected object have one of our custom attribute
				# Remember that the attribute can be there also from a mesh that have noRecurse
				# connections with any compounds because we deleted the history
				if  hasAttribute == True:
				
					#Use temp variables for having easy reading
					currentType = queryResult[1]
					currentAttribute = queryResult[2]
					
					
					#Redefine our var as a list
					self.orderedCompoundsList = []
					self.currentCompoundsType = currentType
					
					
					#Check the first compound of the chain..
					
					# If is of the type in or multi check the source connections..
					# We use the same query for now 
					if currentType == 'in' or currentType == 'multi':
						# The mesh that are hmGenerated, since we are adding the connection to the shape
						shape = cmds.listRelatives(currentSelection, shapes=True)[0]
						currentCompound = cmds.listConnections('%s.%s' % (shape, currentAttribute), s=True, d=False, type = 'hmCompoundNode')

						# currentCompound = cmds.listConnections('%s.%s' % (currentSelection, currentAttribute), s=True, d=False, type = 'hmCompoundNode')

					# If is of the type out just check the outgoing connections..
					if currentType == 'out':
						# On the mesh that have are start mesh, we find the attribute connected on the transform
						currentCompound = cmds.listConnections('%s.%s' % (currentSelection, currentAttribute), d=True, type = 'hmCompoundNode')
					# # If it's multi check the source and destination connections..
					# if currentType == 'multi':
						# currentCompound = cmds.listConnections('%s.%s' % (currentSelection, currentAttribute), s=True, d=False, type = 'hmCompoundNode')
					
						
					# If we actually have found something connected, go ahead with our compound searching..
					if currentCompound != None:
						self.orderedCompoundsList.append(currentCompound[0])				
						# Find all the connected compounds
						self.getAllRelatedCompounds(currentCompound[0])						

						
						
						
					# If we haven't found any connection we set the current compound as none to skip
					# fhurter queryes and just deactivate the ui
					else:
						self.orderedCompoundsList = None 
						
					# print 'current compound', currentCompound
						
						
				#======== End compound searching ======================================================
				
				
				
				#======================================================
				#======== Option Menu Update ==========================
				#======================================================
				if self.orderedCompoundsList != None:
					# print 'found compounds in list: ', self.orderedCompoundsList
					# ==== Activate the attributes
					cmds.frameLayout( self.optionsFrame, edit=True, enable = True)
					

					# # ==== Get related orderToSplit 
					# orderToSplit = cmds.listConnections('%s.inCurves' % currentCompound[0], s=True, d=False, type = 'hmOrderToSplitNode') 
					
					# # ==== Get related hmOffset
					# offsetCurveNode = cmds.listConnections('%s.inCurves' % currentCompound[0], s=True, d=False, type = 'hmOffsetCurveNode') 
					
					# print 'related orderToSplit: ', orderToSplit, 'related hmOffset:', offsetCurveNode
					
					# # ==== Connect the current compound to the interface
					# if orderToSplit:
						# self.hardMeshEditContent.connectCompoundNode( currentCompound[0], orderToSplit[0] )
						
					# if offsetCurveNode:
						# self.hardMeshEditContent.connectCompoundNode( currentCompound[0], offsetCurveNode[0] )
					
					# ======= Connect the current compound and related order to split or offset curve to the UI
					orderOrOffset = self.getRelatedOffsetOrOrderToSplit(currentCompound[0])




					# This method will update the UI
					self.hardMeshEditContent.connectCompoundNode( currentCompound[0], orderOrOffset )
					
					


					
					#======= Delete current UI items, if present
					if len(self.compoundMenuItems) != 0:
						cmds.deleteUI( self.compoundMenuItems )
						#Clear the list
						# print 'cancellato', self.compoundMenuItems
						del self.compoundMenuItems[:]
					
					#======== Update the node selector	
					for x in range(len(self.orderedCompoundsList)):
						# print 'imthaloop'
						# print 'adding: ', self.orderedCompoundsList[x]
						item = cmds.menuItem( label = '%s  %s' % (self.currentCompoundsType, self.orderedCompoundsList[x]), parent = self.compoundsOptionMenu)
						self.compoundMenuItems.append( item )	
					
					
					#======================================================
					#======== Data Storing and finalization ===============
					#======================================================
					# Set the current compound on the option menu
					currentIndex = self.orderedCompoundsList.index( currentCompound[0] ) +1 #Compensate for the 1 starting indices of optionMenu
					cmds.optionMenu( self.compoundsOptionMenu, edit=True, select= currentIndex ) 
					#Store the currently displayed compound
					self.storePreviewSelectedCompound()
					# Store the out mesh
					self.storeCurrentOutMesh()
					# Compile the list of currents attributes for the smooth
					self.getSmoothAttributes()
					# And change curves color..
					self.changeCurvesColor()
					
					
					
				else:
					#Deactivate again the attributes
					self.deactivateAttributesUi()
					#Remove possible curves color override
					self.restoreCurvesColor()
				
				
				
			else:
				#Deactivate again the attributes
				self.deactivateAttributesUi()
				#Remove possible curves color override
				self.restoreCurvesColor()

		
		# ===============================
		# ========= WARNING =============
		# ===== Close the new undo chunk
		cmds.undoInfo(closeChunk=True)
		# ===============================	


		
	def getAllRelatedCompounds(self, inputCurrentCompound ):
		'''
		Following the connections forward and backward, we discover all the related compound nodes starting from one
		'''
		#Check the connections...
		# Forward
		currentCompound = inputCurrentCompound

		# print 'sono nel getAllRelatedCompounds'
		
		while (currentCompound != None):
			#Make the last compound as "selected" and query a different attribute
			currentSelection = currentCompound
			#Control the destinations attributes
			currentCompoundList = self.getCompoundToCompoundConnection(currentSelection, 'forward')
			# currentCompoundList = cmds.listConnections('%s.%s' % (currentSelection, gVar.outCompoundToMesh), s=False, d=True, type = 'hmCompoundNode') 
			# print 'foreward connections: ', currentCompoundList
			#If a connection has been found, add it to our list
			if currentCompoundList != None:
				currentCompound = currentCompoundList[0]
				self.orderedCompoundsList.append(currentCompound)
			else:
				currentCompound = None 	
				
				
		# Backward
		currentCompound = inputCurrentCompound
		
		while (currentCompound != None):	
			#Make the last compound as "selected" and query a different attribute
			currentSelection = currentCompound
			#Control the source attributes	
			currentCompoundList = self.getCompoundToCompoundConnection(currentSelection, 'backward')
			# currentCompoundList = cmds.listConnections('%s.%s' % (currentSelection, gVar.outCompoundToMesh), s=True, d=False, type = 'hmCompoundNode')
			# print 'backward connections: ', currentCompoundList			
			# print 'backward connections: ', currentCompoundList
			if currentCompoundList != None:
				currentCompound = currentCompoundList[0]
				if currentCompound not in self.orderedCompoundsList:
					# Insert always in front for having always an ordered list
					self.orderedCompoundsList.insert(0, currentCompound)
			else:
				currentCompound = None 			
		# End of search

		# print self.orderedCompoundsList

	def getCompoundToCompoundConnection( self,inputCompound, direction ):
		'''
		Get the connected compound in the chosen direction
		'''

		# print 'sono nel getCompoundToCompoundConnection'

		if direction == 'forward':
			# print 'forward'
			# return cmds.listConnections('%s.%s' % (inputCompound, gVar.outCompoundToMesh), s=False, d=True, type = 'hmCompoundNode') 
			return cmds.listConnections('%s.%s' % (inputCompound, gVar.previewAndNextCompound), s=False, d=True, type = 'hmCompoundNode') 


		if direction == 'backward':
			# print 'backward'
			# return cmds.listConnections('%s.%s' % (inputCompound, gVar.outCompoundToMesh), s=True, d=False, type = 'hmCompoundNode')
			return cmds.listConnections('%s.%s' % (inputCompound, gVar.previewAndNextCompound), s=True, d=False, type = 'hmCompoundNode')
			
		# print 'end of searching'
		
	def storePreviewSelectedCompound(self, *Args):
		'''
		This method stores the compound right before the user changes its selection...
		'''
		#Query the selection
		id = cmds.optionMenu( self.compoundsOptionMenu, query=True, select=True) -1 #compensate for the list
		#Store the compound 
		self.lastCompound = self.orderedCompoundsList[id] 
		# print 'last compound was', self.lastCompound
	
	def getSmoothAttributes(self):
		'''
		Given a mesh, we compile a list containing the smooth attributes for a global smooth editing
		We skip the mesh attribute coming from a mesh that is already a result of an operation
		'''
		#Clean any preview attribute
		del self.currentSmoothAttributes[:]
		
		
		for compound in self.orderedCompoundsList:
			#Get the connected meshes
			meshA = cmds.listConnections('%s.%s' % (compound, gVar.inMeshForCompoundA), d=True) [0]
			meshB = cmds.listConnections('%s.%s' % (compound, gVar.inMeshForCompoundB), d=True) [0]
			meshList = [meshA, meshB]
			#Check that the mesh is not from a network..
			for mesh in meshList:
				queryMesh = uiUtils.queryMeshType( mesh )
				#print queryMesh
				#If the mesh is of a valid type
				if queryMesh[1] == 'out':
					self.currentSmoothAttributes.append( '%s.%s' % (mesh, gVar.smoothAttrName) )
					#print self.currentSmoothAttributes
					
	def getRelatedOffsetOrOrderToSplit(self, compoundToQueryFrom):
		'''
		This method search for an order to split or a offsetCurve starting from the input compound.
		This is needed since we are still dealing with 2 kind of networks, the first one messy and with maya offsets.
		The second one much more elegant and with hmOffset.
		'''
		# ==== Get related orderToSplit 
		orderToSplit = cmds.listConnections('%s.inCurves' % compoundToQueryFrom, s=True, d=False, type = 'hmOrderToSplitNode') 
		
		# ==== Get related hmOffset
		offsetCurveNode = cmds.listConnections('%s.inCurves' % compoundToQueryFrom, s=True, d=False, type = 'hmOffsetCurveNode') 
		
		# print 'related orderToSplit: ', orderToSplit, 'related hmOffset:', offsetCurveNode
		
		# ==== Connect the current compound to the interface
		if orderToSplit:
			return orderToSplit[0]
			
		if offsetCurveNode:
			return offsetCurveNode[0]
			
			
			
	def changeCompoundFromMenu(self, *Args):
		'''
		We call this method for changing the interface accordingly to the selected compound in the option menu.
		'''

		#Always change curves color..
		self.changeCurvesColor()
		
		
		# Now, hide the current result, and show the one of the current compound 
		if self.showCurrentResult and self.currentCompoundsType == 'in' or self.showCurrentResult and self.currentCompoundsType == 'multi':
			# print 'starting from compund', self.lastCompound

			if self.currentMesh != None:
				# === Hide the current result
				# print 'hide: ', self.currentMesh
				cmds.setAttr('%s.visibility' % self.currentMesh , 0)
				
				# === Get the selected current compound
				id = self.getCurrentCompoundId() # Get the id of the currentCompound

				# OLD TRANSFORM QUERY
				# connectedOutMesh = cmds.listConnections('%s.%s' % (self.orderedCompoundsList[id], gVar.outCompoundToMesh), d=True, type = 'transform') [0]
				
				# NEW SHAPE METHOD (returns the transform anyway...)
				connectedOutMesh = cmds.listConnections('%s.%s' % (self.orderedCompoundsList[id], gVar.outCompoundToMesh), d=True, type = 'mesh') [0]

				# === And show the related mesh
				cmds.setAttr('%s.visibility' % connectedOutMesh , 1)
				
				# === We save the current as the last one for the next time that we'll call the command
				self.lastCompound = self.orderedCompoundsList[id]
				self.currentMesh = connectedOutMesh
		
		else:
			if self.currentMesh != None:
				id = self.getCurrentCompoundId() # Get the id of the currentCompound
				self.lastCompound = self.orderedCompoundsList[id]

		# print 'connetto all interfaccia:', self.lastCompound
		
		# # Get the orderToSplit related to the last loaded compound
		# orderToSplit = cmds.listConnections('%s.inCurves' % self.lastCompound, s=True, d=False, type = 'hmOrderToSplitNode') [0]
		
		# # === Connect it to the interface		
		# self.hardMeshEditContent.connectCompoundNode( self.lastCompound, orderToSplit)
		
		# ======= Connect the current compound and related order to split or offset curve to the UI
		orderOrOffset = self.getRelatedOffsetOrOrderToSplit( self.lastCompound )
		self.hardMeshEditContent.connectCompoundNode( self.lastCompound, orderOrOffset )
		
				
			
	#================================================
	#============      CURVES COLOR      ============
	#================================================
	
	
	def changeCurvesColor(self):
		'''
		With this method, we change the color override of the related curves.
		This is useful for having immediately clear what are the curves that are contributing to the cut when cascade compounds are present.
		'''
		# Restore curves color before changing
		self.restoreCurvesColor()
				
		# === From current compound
		id = self.getCurrentCompoundId()
		currentCompound = self.orderedCompoundsList[id]
		
		# print 'current compound to change the color to..', currentCompound
		
		# === Get the order to split node...
		orderToSplit = cmds.listConnections('%s.inCurves' % currentCompound, s=True, d=False) #, type = 'hmCompoundNode') 
		
		# We check that the currentCompound have actually an orderToSplitCOnnected...
		if orderToSplit != None:
			# print 'order to split: %s' % orderToSplit[0]
			
			# === From compound get one input curves..
			inCurves = cmds.listConnections('%s.inCurves' % orderToSplit[0], s=True, d=False) #, type = 'hmCompoundNode')
			
			# We check that the orderToSplit have actually some curves connected, if not it's probabli oprhan...
			if inCurves != None:
				# === From inCurves use one to get the related group, and store it 
				curveGrp = cmds.listRelatives(inCurves[0], p=True, f=True)[0]
				#Now we want to change directly the curve transform color, so we get all the children of that group
				self.currentCurves = cmds.listRelatives(curveGrp, c=True, f=True)
				# #And change color of every curve
				for curve in self.currentCurves:
					cmds.setAttr('%s.overrideEnabled' % curve, 1)
					cmds.setAttr('%s.overrideColor' % curve, gVar.activeCurvesColor)

				
			#Now that everything has been set save current as last..
			self.lastCurves = self.currentCurves

	def restoreCurvesColor(self):
		'''
		This mehtod it's called for restoring back to the original color the curves contained in the curves array.
		We mainly use this method for turning off color override when changin between hmCompounds but also
		when the user select nothing we can reset anyway the curves color, and also if the script job is killed..
		'''
		#If we have curves stored with a color override on, turn it off before changing everything else.
		if self.lastCurves != None:
			for curve in self.lastCurves:
				#make sure that these curves are still in scene
				if cmds.objExists(curve):
					#If they are, delete they
					cmds.setAttr('%s.overrideEnabled' % curve, 0)
	
		#Empty the var
		self.lastCurves = None 
	

	
	
	
	
	def getCurrentCompoundId(self):
		'''
		This utility method returns the current compound id
		'''
		#Query the selection and return the ID
		compoundID = cmds.optionMenu( self.compoundsOptionMenu, query=True, select=True) -1 #compensate for the list

		return compoundID
		
	def deactivateAttributesUi(self):	
		'''
		Deactivate the elements of the ui when requested
		'''
		#Deactivate the attributes
		cmds.frameLayout( self.optionsFrame, edit=True, enable = False)
		#Delete the menu items
		cmds.deleteUI(self.compoundMenuItems)
		#Clear the list
		del self.compoundMenuItems[:]	
		#Add empty items
		self.compoundMenuItems.append( cmds.menuItem( label = 'Select A mesh...', parent = self.compoundsOptionMenu ) )
		
		


			
			
	def updateLayers(self):
		'''
		Update all the layers, this will avoid layers/ui mismatch
		'''
		#Handle the result status 
		if self.layerExist(self.outLayerName):
			baseVis = self.getLayerStatus( self.origLayerName )
			if baseVis == True:
				cmds.setAttr( '%s.displayType' % self.outLayerName, 2 )
			else:
				cmds.setAttr( '%s.displayType' % self.outLayerName, 0 )
		
		#Update the check box status
		cmds.checkBox( 'cbBase', edit = True, v = self.getLayerStatus( self.origLayerName ) )
		cmds.checkBox( 'cbResult', edit = True, v = self.getLayerStatus( self.outLayerName ) )
		cmds.checkBox( 'cbCurve', edit = True, v = self.getLayerStatus( self.curvesLayerName ) )
		
	def getLayerStatus( self, layerName ):
		'''
		Get the current layer status for interface, if does not exist return one
		'''
		exist = self.layerExist( layerName )

		if exist == 1:
			return cmds.getAttr( '%s.visibility' % layerName )
		else:
			return 1		
		
	def layerExist( self, layerName ):
		'''
		Return if the provided layer esists. Return 1 if true 0 if false.
		'''
		try:
			cmds.getAttr( '%s.visibility' % layerName )
			return 1
		except:
			#keep the warning turned off.. 
			#cmds.warning('No layer named %s in scene.' % layerName)
			return 0
		
