#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
import maya.mel as mel
# General modules
import hmScripts.UI.hmGlobalVariables as gVar
# Ui Modules
import hmScripts.UI.hmUiCommonUtils as uiUtil
import hmScripts.UI.hmGradientControlFunctions as gradFunct

# Creation modules
import hmScripts.Main.hmSelectionHandling as hmSelect
import hmScripts.Main.hmNodeConnector as hmNodesConnection


reload(gVar)
reload(uiUtil)
reload(gradFunct)
reload (hmSelect)
reload(hmNodesConnection)



class HardMeshOperationOptionsUI():
	def __init__(self, asParameterChanger = False):

		#Tooltips
		self.tooltipArray = {
		'Main Click' : 'Get the blended model in one click!\n',
		'Scale Factor' : 'The precision used in offset computing. Smaller values equals to better results.',
		'Geo Bias' : 'Set to 0 to cut the mesh on the real intersection, set to 1 to move nearest vertices before cutting.',		
		'Sections Spans' : 'Set the number of spans for the blending strips.',
		'Split Refine' : 'Off: cut only on the curve edge intersection. Chord Dist: cut using a tolerance value for following the curve. Strict: cut exactly where is the offset. On Scale: use the global shale factor as a chord dist value.',
		
		'MeshSolver4' : 'Merge the original meshes with the blending strips.\nKeep in mind that this is required for solving multiple hmSetup',
		'MeshSolver5' : 'Hide hmSetup after solve. Leave this on for avoiding \nboolean recomputation in multiple hmSetup solving',
		'MeshSolver6' : 'Delete the hmSetup involved in current solving',

		'Split2' : 'Add points on splitted edge to increase curvature fidelity',
		'Split3' : 'The max points that will be added on the edge',
		'Split4' : 'Minimum distance in world units for adding points on the splitted edge.',

		'Corner Bias' : 'Consider a corner the angles over this degree',

		}



		#============================= FORCED PARAMETERS =========================================
		self.connectedCompound =  None
		self.connectedOrderToSplit = None
		self.connectedOffsetNode = None


		#============================= FORCED PARAMETERS =========================================
		self.forceOperation = 0
		
		#============================= PARAMETER CHANGER =========================================
		#Define here if you want to init the class asParameterChanger, we use this in the smart edit window
		self.asParameterChanger = asParameterChanger
		self.connectedCompound = None
		self.connectedOrderToSplit = None
		self.hmOpt_offsetValA = None
		self.hmOpt_offsetValB = None
		self.meshA = None
		self.meshB = None

		
		
		
		#============================= UI DEFAULT ATTRIBUTES =========================================
		#We define here the default values for the options
		self.hmOpt_defaultScaleFactor = 0.05
		self.hmOpt_defaultOffsetType = 1 #1 hmOffset 2 maya offset
		
		
		self.hmOpt_defaultOperation = 1
		self.hmOpt_defaultSmoothA = 1
		self.hmOpt_defaultSmoothB = 1
		self.hmOpt_defaultOffsetVal = 0.1
		self.hmOpt_defaultCornerBias = 0
		self.hmOpt_defaultGeoBias = 0.0
		self.hmOpt_defaultSections = 5
		
		self.hmOpt_defaultRefine = 2
		self.hmOpt_defaultChordDist = 0.01
		
		self.hmOpt_defaultStrategy = 3
		
		self.hmOpt_defaultSmoothProfile = 1
		# self.hmOpt_defaultFillerRefinment = 0 #Don't know If I'll use it
		self.hmOpt_defaultWeightA = 0.5
		self.hmOpt_defaultWeightB = 0.5
		self.hmOpt_defaultActiveRampedFiller = 0
		self.hmOpt_defaultProfileCurve = [u'0.5,0,2', u'0.5,1,2']
		self.hmOpt_defaultInterpolation = 4
		
		self.hmOpt_defaultRampChord = 0.1
		self.hmOpt_defaultRampOffset = -0.5
		self.hmOpt_defaultRampScale = 1
		self.hmOpt_defaultRampMode = 1  
		
		# Order to split related
		self.hmOpt_defaultBindBias = 0.01
		# self.hmOpt_defaultSequencer = 0
		self.hmOpt_defaultSequencerTolerance = 0.001
		self.hmOpt_defaultRemoveCurl = 0

		# self.hmOpt_defaultAutoMerge = 1
		# self.hmOpt_defaultTolerance = 0.001
		self.hmOpt_defaultComputeNormals = 1



		#============================= UI ATTRIBUTES =========================================
		#We store the values in these list with 2 entry: the first one is the string for the name, and the second one is the value
		self.hmOpt_ScaleFactor = ['hmOpt_defaultScaleFactor', self.hmOpt_defaultScaleFactor ]
		self.hmOpt_offsetType = ['hmOpt_offsetType', self.hmOpt_defaultOffsetType]
		
		
		self.hmOpt_OperationVal = ['hmOpt_OperationVal', self.hmOpt_defaultOperation]	
		self.hmOpt_SmoothValA = ['hmOpt_SmoothValA ', self.hmOpt_defaultSmoothA]
		self.hmOpt_SmoothValB = ['hmOpt_SmoothValB ', self.hmOpt_defaultSmoothB]
		self.hmOpt_offsetVal = ['hmOpt_offsetVal', self.hmOpt_defaultOffsetVal]
		self.hmOpt_CornerBiasVal = ['hmOpt_CornerBiasVal', self.hmOpt_defaultCornerBias ]
		self.hmOpt_GeoBiasVal = ['hmOpt_GeoBiasVal', self.hmOpt_defaultGeoBias]
		self.hmOpt_SectionsVal = ['hmOpt_SectionsVal', self.hmOpt_defaultSections]
		
		self.hmOpt_RefineVal = ['hmOpt_RefineVal', self.hmOpt_defaultRefine]
		self.hmOpt_ChordDistVal = ['hmOpt_MS_ChordDistVal', self.hmOpt_defaultChordDist]
		
		self.hmOpt_StrategyVal = ['hmOpt_MS_StrategyVal', self.hmOpt_defaultStrategy]
		
		self.hmOpt_SmoothProfileVal = ['hmOpt_MS_SmoothProfileVal', self.hmOpt_defaultSmoothProfile]
		# self.hmOpt_FillerRefinmentVal = ['hmOpt_FillerRefinmentVal', self.hmOpt_defaultFillerRefinment]
		self.hmOpt_WeightAVal = ['hmOpt_WeightAVal', self.hmOpt_defaultWeightA]
		self.hmOpt_WeightBVal = ['hmOpt_WeightBVal', self.hmOpt_defaultWeightB]
		self.hmOpt_ActiveRampedFillerVal  = ['self.hmOpt_ActiveRampedFillerVal', self.hmOpt_defaultActiveRampedFiller]
		self.hmOpt_ProfileCurveVal = ['hmOpt_ProfileCurveVal', self.hmOpt_defaultProfileCurve]
		self.hmOpt_InterpolationVal = ['hmOpt_InterpolationVal', self.hmOpt_defaultInterpolation]
		
		self.hmOpt_RampChordVal = ['self.hmOpt_RampChordVal', self.hmOpt_defaultRampChord]
		self.hmOpt_RampOffsetVal = ['self.hmOpt_RampOffsetVal', self.hmOpt_defaultRampOffset]
		self.hmOpt_RampScaleVal = ['self.hmOpt_RampScaleVal', self.hmOpt_defaultRampScale]
		self.hmOpt_RampModeVal = ['self.hmOpt_RampModeVal', self.hmOpt_defaultRampMode]
		
		
		# Order to split related
		self.hmOpt_BindBiasVal = ['self.hmOpt_BindBiasVal', self.hmOpt_defaultBindBias]
		# self.hmOpt_SequencerVal = ['self.hmOpt_SequencerVal', self.hmOpt_defaultSequencer]
		# self.hmOpt_SequencerToleranceVal = ['self.hmOpt_SequencerToleranceVal', self.hmOpt_defaultSequencerTolerance]
		self.hmOpt_RemoveCurlVal = ['self.hmOpt_RemoveCurlVal', self.hmOpt_defaultRemoveCurl]
		
		
		# self.hmOpt_AutoMergeVal = ['hmOpt_AutoMergeVal',self.hmOpt_defaultAutoMerge]
		# self.hmOpt_ToleranceVal = ['hmOpt_ToleranceVal', self.hmOpt_defaultTolerance]
		self.hmOpt_ComputeNormalsVal = ['hmOpt_ComputeNormalsVal', self.hmOpt_defaultComputeNormals]
		



		#============================= GRADIENT FUNCTIONS =========================================
		# =========== Gradient functions class, handle the gradient edits
		self.gFunct = gradFunct.GradientControl( self.hmOpt_ProfileCurveVal, self.asParameterChanger )
		



		#============================= OPTIONS VAR =========================================
		#If we initialize the class as parameter changer ther's no reason for storing option vars..
		if self.asParameterChanger == False:
			self.optionsList = [ self.hmOpt_ScaleFactor,
			self.hmOpt_OperationVal, self.hmOpt_SmoothValA, self.hmOpt_SmoothValB, self.hmOpt_offsetVal, self.hmOpt_CornerBiasVal, self.hmOpt_GeoBiasVal, self.hmOpt_SectionsVal,
			self.hmOpt_RefineVal, self.hmOpt_ChordDistVal, self.hmOpt_StrategyVal, self.hmOpt_SmoothProfileVal, self.hmOpt_WeightAVal, self.hmOpt_WeightBVal, self.hmOpt_ActiveRampedFillerVal, 
			self.hmOpt_ProfileCurveVal, self.hmOpt_InterpolationVal, self.hmOpt_RampChordVal, self.hmOpt_RampOffsetVal, self.hmOpt_RampScaleVal, self.hmOpt_RampModeVal, 
			self.hmOpt_BindBiasVal, self.hmOpt_RemoveCurlVal, self.hmOpt_ComputeNormalsVal 	
			]
		
			# #For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
			uiUtil.optionVarInit( self.optionsList ) 
			# uiUtil.optionVarRemove(self.optionsList)

	def setForcedAttributes(self, forceOperation):
		'''
		We use this method for forcing some attributes for UI purposes with the icons on the main window.
		'''
		self.forceOperation = forceOperation
		
		#If we want to build the window with a forced operation
		if self.forceOperation == 1:
				self.hmOpt_OperationVal = ['hmOpt_OperationValUn', 1 ]
		elif self.forceOperation == 2:	
				self.hmOpt_OperationVal = ['hmOpt_OperationValAB', 2 ]
		elif self.forceOperation == 3:	
			self.hmOpt_OperationVal = ['hmOpt_OperationValBA', 3 ]
		elif self.forceOperation == 4:	
				self.hmOpt_OperationVal = ['hmOpt_OperationValInt', 4 ]
		elif self.forceOperation == 5:	
				self.hmOpt_OperationVal = ['hmOpt_OperationValPAB', 5 ]				
		elif self.forceOperation == 6:	
				self.hmOpt_OperationVal = ['hmOpt_OperationValPBA', 6 ]			

			
	def initCompoundAttr(self):
		'''
		THis method is called for building the attributes strings for the current compound node
		'''
		
		# These two no longer exist.. attribute moved directly on the meshes for speed reasons
		# self.nodeSmoothAttrNameA = '%s.%s_0' % (self.connectedCompound, gVar.smoothAttrName)
		# self.nodeSmoothAttrNameB = '%s.%s_1' % (self.connectedCompound, gVar.smoothAttrName) 
		
		
		#Get the connected meshes
		self.meshA = cmds.listConnections('%s.%s' % (self.connectedCompound, gVar.inMeshForCompoundA), d=True) [0]
		self.meshB = cmds.listConnections('%s.%s' % (self.connectedCompound, gVar.inMeshForCompoundB), d=True) [0]
		#We use the attributes of the smooth directly on the meshes
		self.nodeSmoothAttrNameA = '%s.%s' % (self.meshA, gVar.smoothAttrName)
		self.nodeSmoothAttrNameB = '%s.%s' % (self.meshB, gVar.smoothAttrName) 
		
		
		#Again search in the node...
		# These two were causing problems in the update
		# self.nodeOffsetAttrNameA = '%s.%s_0' % (self.connectedCompound, gVar.offsetAttrName)
		# self.nodeOffsetAttrNameB = '%s.%s_1' % (self.connectedCompound, gVar.offsetAttrName)



		self.nodeCornerBias = '%s.%s' % (self.connectedCompound, gVar.cornerBias)
		self.nodeGeometryBias = '%s.%s' % (self.connectedCompound, gVar.geometryBias)		
		self.nodeSectionSpans = '%s.%s' % (self.connectedCompound, gVar.sectionSpans)
		
		self.nodeRefineEdge = '%s.%s' % (self.connectedCompound, gVar.edgeSubd)
		self.nodeChordDist = '%s.%s' % (self.connectedCompound, gVar.tolerance)
		
		self.nodeFillStrategy = '%s.%s' % (self.connectedCompound, gVar.fillStrategy)
		# self.nodeRefineFiller = '%s.%s' % (self.connectedCompound, gVar.refinement)
		
		self.nodeSmoothFiller = '%s.%s' % (self.connectedCompound, gVar.smoothFiller)
		
		self.nodeWeightA = '%s.%s' % (self.connectedCompound, gVar.weightA)
		self.nodeWeightB = '%s.%s' % (self.connectedCompound, gVar.weightB)
		
		self.nodeUseRamp = '%s.%s' % (self.connectedCompound, gVar.activeRampedFiller)
		
		self.nodeRampChord = '%s.%s' % (self.connectedCompound, gVar.rampChordDist)
		self.nodeRampOffset = '%s.%s' % (self.connectedCompound, gVar.rampOffset)
		self.nodeRampScale = '%s.%s' % (self.connectedCompound, gVar.rampScale)
		self.nodeRampMode = '%s.%s' % (self.connectedCompound, gVar.rampMode)
		
		#Gradient options, check the gradient control with the attribute..
		self.nodeRampedFiller = '%s.%s' % (self.connectedCompound, gVar.rampedFiller)

		
		#Static ID ramp attr
		self.nodeRampedFiller_InterpONE = '%s.%s[1].%s' % (self.connectedCompound, gVar.rampBaseId, gVar.rampInt)
		self.nodeRampedFiller_ValONE = '%s.%s[1].%s' % (self.connectedCompound, gVar.rampBaseId, gVar.rampVal)
		self.nodeRampedFiller_PosONE = '%s.%s[1].%s' % (self.connectedCompound, gVar.rampBaseId, gVar.rampPos)
		
		
		# self.nodeAutoMerge = '%s.%s' % (self.connectedCompound, gVar.autoMergeResult)
		# self.nodeMergeDist = '%s.%s' % (self.connectedCompound, gVar.mergeDist)
		self.nodeComputeNormals = '%s.%s' % (self.connectedCompound, gVar.computeNormals)
		self.nodeCompoundScaleFactor = '%s.%s' % (self.connectedCompound, gVar.scaleFactor)
		
		#Order To split Parameters
		self.nodeBindBias = '%s.%s' % (self.connectedOrderToSplit, gVar.bindBias)
		self.nodeRemoveCurl = '%s.%s' % (self.connectedOrderToSplit, gVar.removeCurl)
		self.nodeCurveSequencer = '%s.%s' % (self.connectedOrderToSplit, gVar.curveSequencer)
		self.nodeSequenceBias = '%s.%s' % (self.connectedOrderToSplit, gVar.sequenceBias)
		
		#Offset node Parameters
		self.nodeOperationType = '%s.%s' % (self.connectedOffsetNode, gVar.booleanOperation)
		self.nodeOffsetRemoveCurl = '%s.%s' % (self.connectedOffsetNode, gVar.removeCurl)
		self.nodeOffsetScaleFactor = '%s.%s' % (self.connectedOffsetNode, gVar.scaleFactor)
		self.nodeOffsetAttrNameA = '%s.%s' % (self.connectedOffsetNode, gVar.offsetA)
		self.nodeOffsetAttrNameB = '%s.%s' % (self.connectedOffsetNode, gVar.offsetB)



	def createUiElements(self):
		'''
		This function implements the shared UI elements for mesh setup.
		We will use the same options both on the hmDockUi and from the floating panels options, for this reason we should avoid name conflicts of the widgets...
		'''

		#Variable number of columns, 2 for creation UI, 3 for parameter changer with the "apply" button
		columns = 2
		
		if self.asParameterChanger:
			columns = 3 #If parameter changer, 3 columns for apply buttton
		else:
			# ====== DEVELOPING VAR 	
			if gVar.develop == 1:
				columns +=1	
				
		cmds.separator( height = 1, style='in' )
		self.firstRow = cmds.rowLayout('first row', nc = columns )
			
		# ====== DEVELOPING VAR 
		if gVar.develop == 1 and self.asParameterChanger == False:
			columns +=1
			self.offTypeOperation = cmds.radioButtonGrp('rOffType', labelArray2=['hmOffset', 'mayaOffset'], numberOfRadioButtons=2, columnWidth = ([1, 80], [2,100]), select = self.hmOpt_offsetType[1], en=True )
			
			
		
		self.omOperation = cmds.optionMenu(label = 'operation') #, changeCommand = self.changeValueCommand )
		cmds.menuItem( label = gVar.operationA, data = 0 )
		cmds.menuItem( label = gVar.operationB, data = 1 )
		cmds.menuItem( label = gVar.operationC, data = 2 )
		cmds.menuItem( label = gVar.operationD, data = 3 )
		cmds.menuItem( label = gVar.operationE, data = 4 )
		cmds.menuItem( label = gVar.operationF, data = 5 )

		#Set the value
		cmds.optionMenu( self.omOperation, edit = True, select = self.hmOpt_OperationVal[1] )
		self.ifgSmoothAB = cmds.intFieldGrp( numberOfFields=2, label='Smooth Mesh', columnWidth = [(1,100), (2,30), (3,30)], adj = 1, value1= self.hmOpt_SmoothValA[1], value2= self.hmOpt_SmoothValB[1]) #, changeCommand = self.changeValueCommand )
		#If we are forcing the operation, like the option boxes from hmWindow, lock the option menu..
		if self.forceOperation != 0:
			cmds.optionMenu(self.omOperation, edit = True, en= False )
		
		#=================== if parameter changer =====================================
		if self.asParameterChanger:
			self.smoothButon = cmds.button(label = 'Apply', command = self.changeSmoothValues) #We put here a button for this value change, that seems to be very slow..
		#==============================================================================
		


		cmds.setParent( '..' )
		
		# Separator time
		cmds.separator( height = 1, style='in' )





		#======================SCALE FACTOR ==========================
		#That drives the compound scale too if parameter changer...
		if gVar.globalScaleFactor == 1:
			self.fsScaleFactor = cmds.floatSliderGrp( label ='Scale Factor', field=True, columnWidth= ([2, 50], [3,100]) , precision = 5, value = self.hmOpt_ScaleFactor[1], max = 1, min = 0.0001, ann = self.tooltipArray['Scale Factor'], cc = self.updateScaleOnCompound) 
			cmds.separator( height = 1, style='in' )


		#=================== if parameter changer =====================================
		maxInitOffset = 2
		absoluteMax = 100

		if self.asParameterChanger:


			# Create the UI elements and leave visible only the relevant ones
			self.doubleOffsetLayout = cmds.columnLayout( adj = 1, visible = False)
			cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )
			self.fsOffsetA = cmds.floatSliderGrp( label ='OffsetA', field=True, precision = 3, columnWidth= ([2, 50], [3,100]),
												  value = self.hmOpt_offsetVal[1] , max = maxInitOffset, min = 0, fieldMaxValue =  absoluteMax)
			cmds.button(label = 'Merge Sides', c=self.mergeOffset )	
			cmds.setParent('..')
			cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )
			self.fsOffsetB = cmds.floatSliderGrp( label ='OffsetB', field=True, precision = 3, columnWidth= ([2, 50], [3,100]),
												  value = self.hmOpt_offsetVal[1] , max = maxInitOffset, min = 0, fieldMaxValue =  absoluteMax)	
			cmds.button(label = 'Merge Sides', visible = False )	
			cmds.setParent('..')

			# Back to column
			cmds.setParent('..')
			


			self.singleOffsetLayout = cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )
			# Check if we have to display one or wto slider
			self.fsOffset = cmds.floatSliderGrp( label ='Offset', field=True, precision = 3, columnWidth= ([2, 50], [3,100]),
												 value = self.hmOpt_offsetVal[1] , max = maxInitOffset, min = 0, fieldMaxValue =  absoluteMax )	
			cmds.button(label = 'Split Sides', c=self.splitOffset )
			cmds.setParent('..')
		#==============================================================================
		else:
			self.fsOffset = cmds.floatSliderGrp( label ='Offset', field=True, precision = 3, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_offsetVal[1] , max = maxInitOffset, min = 0 )	
		
		
		
		
		
		
		#==============================================================================
		#The visibility of corner bias is on only for the develop...
		self.fsGlobCorBias = cmds.floatSliderGrp( label ='Corner Bias', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_CornerBiasVal[1], max=180, ann =  self.tooltipArray['Corner Bias'], en=False, visible = gVar.develop )
		
		# These two are always visible
		self.fsGlobGeoBias = cmds.floatSliderGrp( label ='Geometry Bias', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_GeoBiasVal[1], precision = 3, max = 1, min = 0, ann =  self.tooltipArray['Geo Bias'] )	
		self.isGlobSections = cmds.intSliderGrp( label ='Sections Spans', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_SectionsVal[1], min=2, ann =  self.tooltipArray['Sections Spans'] )
		
		

		
		if gVar.globalScaleFactor == 1:
			# Create the global tolerance that drives also the chord dist...
			if self.asParameterChanger:
				#That drives the compound scale too if parameter changer...
				self.isAdaptVal = cmds.floatSliderGrp( label ='Chord Dist', field=True, columnWidth= ([2, 50], [3,100]) , precision = 5, value = self.hmOpt_ChordDistVal[1], max = 1, min = 0, ann = self.tooltipArray['Split4'], cc = self.updateScaleOnCompound)
				# self.setRefineOptStatus()
			else:
				# Create the chord dist parameter
				self.isAdaptVal = cmds.floatSliderGrp( label ='Chord Dist', field=True, columnWidth= ([2, 50], [3,100]) , precision = 5, value = self.hmOpt_ChordDistVal[1], max = 1, min = 0, ann = self.tooltipArray['Split4'])	
				# self.setRefineOptStatus()
		else:
			# Create the chord dist parameter
			self.isAdaptVal = cmds.floatSliderGrp( label ='Chord Dist', field=True, columnWidth= ([2, 50], [3,100]) , precision = 5, value = self.hmOpt_ChordDistVal[1], max = 1, min = 0, ann = self.tooltipArray['Split4'])	
			# self.setRefineOptStatus()




		#======================SCALE FACTOR ==========================
		# if gVar.globalScaleFactor == 1:
		# 	self.omRefEdge = cmds.optionMenuGrp( label = 'Split Refine', changeCommand = self.setRefineOptStatus, ann =  self.tooltipArray['Split Refine'] )
		# 	cmds.menuItem( label = gVar.edgeSubdA, data = 0 )
		# 	cmds.menuItem( label = gVar.edgeSubdB, data = 1 )
		# 	cmds.menuItem( label = gVar.edgeSubdC, data = 2 )
		# 	cmds.menuItem( label = gVar.edgeSubdD, data = 3 ) # when we are on scale, actually we use the chord dist with the same val of scale...

		# else:
		# 	self.omRefEdge = cmds.optionMenuGrp( label = 'Split Refine', changeCommand = self.setRefineOptStatus, ann =  self.tooltipArray['Split Refine'] )
		# 	cmds.menuItem( label = gVar.edgeSubdA, data = 0 )
		# 	cmds.menuItem( label = gVar.edgeSubdB, data = 1 )
		# 	cmds.menuItem( label = gVar.edgeSubdC, data = 2 )


		#Set the value
		# print 'devo selezionare:', self.hmOpt_RefineVal[1] 
		# Weird error...happens that this value is saved as 0...
		# if gVar.globalScaleFactor ==1:
		# 	if self.hmOpt_RefineVal[1] == 0:
		# 		self.hmOpt_RefineVal[1] = 4


		# cmds.optionMenuGrp( self.omRefEdge, edit = True, select = self.hmOpt_RefineVal[1] )


		self.omRefEdge = cmds.optionMenuGrp( label = 'Split Refine', changeCommand = self.setRefineOptStatus, ann =  self.tooltipArray['Split Refine'] )
		cmds.menuItem( label = gVar.edgeSubdA, data = 0 )
		cmds.menuItem( label = gVar.edgeSubdB, data = 1 )
		cmds.menuItem( label = gVar.edgeSubdC, data = 2 )
		cmds.optionMenuGrp( self.omRefEdge, edit = True, select = self.hmOpt_RefineVal[1] )




		

		
		
		#=========================================>>>>> BLENDING TAB <<<<<<<<============================================================		
		self.blendingTab = cmds.frameLayout( label = 'Blending' , collapsable= True , collapse = False, expandCommand = self.controlRampStatus, mh=10, mw=10 ) #bgs=True maya 2015 non lo sopporta..
		
		
		#======= Blend Strategy
		#The blend strategy is visible only if we are in develop
		self.omStrategy = cmds.optionMenu(  label = 'Blend Strategy', visible = gVar.develop ) #, changeCommand = self.changeValueCommand )
		cmds.menuItem( label = 'Standard', data = 7 )
		cmds.menuItem( label = 'Equidistant', data = 4)
		cmds.menuItem( label = 'Experimental', data = 10 )
		# self.cbRefFiller = cmds.checkBox( label = 'Refine Blend', value =self.hmOpt_FillerRefinmentVal[1], ann = self.tooltipArray['Split2'], changeCommand = self.changeValueCommand )

		#Set the value
		cmds.optionMenu( self.omStrategy, edit = True, select = self.hmOpt_StrategyVal[1] )
		
		#====== Profile Interpolation
		cmds.separator( height = 1, style='in' )
		self.cbInterpProfile = cmds.checkBox( label = 'Interpolate Blend', value = self.hmOpt_SmoothProfileVal [1], changeCommand = self.setWeightsStatus )
		self.fsWeightA = cmds.floatSliderGrp(label='weight A', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_WeightAVal[1], min=0, max=1)
		self.fsWeightB = cmds.floatSliderGrp(label='weight B', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_WeightBVal[1], min=0, max=1)
		# Set the visibility of weights
		self.setWeightsStatus()
		
		# Create the ramp check box
		self.cbActiveRamp = cmds.checkBox( label = 'Use Ramp', value = self.hmOpt_ActiveRampedFillerVal[1], onc = self.enableProfileControl,  ofc = self.disableProfileControl)

		
		
		
		
		
		
		#============================================================================================================
		#====== Gradient Layout ====== 
		self.gradFrame = cmds.frameLayout( label = 'Ramp Controls', visible=self.hmOpt_ActiveRampedFillerVal[1])
		
		# #Build controls...

		# Visible if in develop
		if gVar.develop == 1:
			self.fsRampChord = cmds.floatSliderGrp(label='Ramp Chord Resolution', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampChordVal[1], min=0.001, max=1, precision = 4)
			self.cbRampMode = cmds.checkBox( label = 'Parallel Move', value = self.hmOpt_RampModeVal[1], ann = self.tooltipArray['Split2'] )
		#Hidden for the current build
		else:	
			self.fsRampChord = cmds.floatSliderGrp(label='Ramp Chord Resolution', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampChordVal[1], min=0.001, max=1, precision = 4, visible = False)
		
		self.fsRampOffset = cmds.floatSliderGrp(label='Ramp Offset', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampOffsetVal[1], min=-10, max=10, precision = 3)
		self.fsRampScale = cmds.floatSliderGrp(label='Ramp Scale', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampScaleVal[1], min=-10, max=10, precision = 3)
		




		self.gradLayout = cmds.rowLayout(nc=3, columnWidth3=[80,90, 5000], columnAttach3 = ['both', 'both', 'left'], columnOffset3 = [3,3,3], parent = self.gradFrame)


		cmds.columnLayout(adjustableColumn = True, rs=8, cal='right')
		cmds.text('Interpolation:')
		cmds.text('Position:')
		cmds.text('Value:')
		cmds.setParent('..') #Change the parent again to row
		
		self.gradientAttrParent = cmds.columnLayout(adjustableColumn = True, rs=3)
		self.omInterp = cmds.optionMenu( cc = self.gFunct.cangeCurrentPoint )
		self.ffInterpPos = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
		self.ffInterpVal  = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
		cmds.setParent('..') #Change the parent again to row
		
		cmds.menuItem( label = 'Linear' )
		cmds.menuItem( label = 'Smooth' )
		cmds.menuItem( label = 'Spline' )
		cmds.menuItem( label = 'None' )		
		

		#===== Third column, in the editing mode it should be probably changed with the gradient control with attr	
		cmds.columnLayout(adjustableColumn = True, rs=3)

		self.gradientParent = cmds.rowLayout( nc=2, columnWidth2=[100, 500])			
		self.gcProfileCurve = cmds.gradientControlNoAttr( 'falloffCurve', h=90, w = 200,  optionVar=self.hmOpt_ProfileCurveVal[0], currentKeyChanged = self.gFunct.updateGradValuesUi, dragCommand = self.gFunct.updateGradValuesUi)
		self.editGradButton = cmds.button(label= '>', h=90, w= 20, c=self.editGradientAttributeInWindow, visible = self.asParameterChanger)

		
		# =========== Gradient functions class, set the controls
		self.gFunct.setControls( self.gcProfileCurve, self.omInterp, self.ffInterpPos, self.ffInterpVal )





		cmds.setParent('..') #Change the parent again to row		
		cmds.setParent('..') #Change the parent again to row
		#============================================================================================================
		#============================================================================================================
		
		
		
		
		#Change the parent to..				
		cmds.setParent('..')
		
		
		cmds.rowLayout( nc=2, columnWidth2=[100, 500])
		cmds.text('Curve presets:')
		cmds.rowLayout(nc=5)
		self.button1 = cmds.symbolButton(image= 'gradientControl_icon_01.png', c= self.gFunct.setGradientControlTo1 )
		self.button2 = cmds.symbolButton(image= 'gradientControl_icon_02.png', c= self.gFunct.setGradientControlTo2 )
		self.button3 = cmds.symbolButton(image= 'gradientControl_icon_03.png', c= self.gFunct.setGradientControlTo3 )
		self.button4 = cmds.symbolButton(image= 'gradientControl_icon_04.png', c= self.gFunct.setGradientControlTo4 )
		self.button4 = cmds.symbolButton(image= 'gradientControl_icon_05.png', c= self.gFunct.setGradientControlTo5 )

			
			
			
			
			
			
		#=========================================>>>>> ADVANCED TAB <<<<<<<<============================================================	
		#============================================================================================================
		
		
		# ====== DEVELOPING VAR 	
		if gVar.develop == 1:	
			#Change the parent to..
			cmds.setParent(self.blendingTab)
			cmds.setParent('..')
			

			
			self.advancedTab = cmds.frameLayout( label = 'Extra' , collapsable= True , collapse = True , mh=10, mw=10 )

			# row = cmds.rowLayout( nc = 4, columnAttach4 = ['both', 'both', 'both', 'both'], columnOffset4 = [0,0,30,3] )
			# cmds.text('Bind Bias')
			# self.ffBindBias = cmds.floatField( value = self.hmOpt_BindBiasVal[1], precision = 5 )
			# cmds.separator( height = 20, hr=False, style='in' )
			# self.cbRemCurl = cmds.checkBox( label = 'Remove Curl', v = self.hmOpt_RemoveCurlVal[1], ann = self.tooltipArray['MeshSolver4'])
			# Exit from row layout
			# cmds.setParent('..')



			cmds.separator( height = 1, style='in' )

			row = cmds.rowLayout( nc = 3, columnAttach4 = ['both', 'both', 'both', 'both'], columnOffset4 = [0,0,30,3] )
			# self.cbAutoMerge = cmds.checkBox( label = 'Auto Merge', v = self.hmOpt_AutoMergeVal[1], onc = self.enableGlobMerge, ofc = self.disableGlobMerge, ann = self.tooltipArray['MeshSolver4'])
			# cmds.text('Merge Tolerance')
			# self.ffGlobTol = cmds.floatField(  value = self.hmOpt_ToleranceVal[1], precision = 5, en=self.hmOpt_AutoMergeVal[1] )
			self.cbRemCurl = cmds.checkBox( label = 'Remove Curl', v = self.hmOpt_RemoveCurlVal[1])
			cmds.separator( height = 20, hr=False, style='in' )
			self.cbComputeNorm = cmds.checkBox( label = 'Compute Normals', v = self.hmOpt_ComputeNormalsVal [1], ann = self.tooltipArray['MeshSolver5'])


			# Reset the parent for the next controls
			cmds.setParent('..')
			cmds.setParent('..')
			cmds.setParent('..')
			cmds.setParent('..')
		else:
			#Change the parent to..
			cmds.setParent(self.blendingTab)
			#cmds.setParent(self.firstRow)	
			cmds.setParent('..')	
			cmds.setParent('..')	
			cmds.setParent('..')	

		
		
	#===========================================================================================================	
	#================================ COMMANDS ========================================================
	#===========================================================================================================

	def performCommand(self, *Args ):
		'''
		Perform the command
		'''
		createOperation = True
		
		#Init the sel handling
		selection = hmSelect.HmSelectionHandling()
		#Set custom messages
		selection.setErrorMessageIfZero('Please select 2 meshes to operate on.')
		selection.setErrorIfSmallThanMin('Please select 2 meshes to operate on.')
		selection.setErrorMessageIfOutLimit('Please select only 2 objects. You can have multiple shells for each mesh.')
		#Validate the selection and put the result on a var 

		# If we support multimesh
		if gVar.multimesh:
			sel = selection.validate(2,None,'mesh')

			meshA = sel[0]
			meshB = sel[1:]

		else:
			sel = selection.validate(2,2,'mesh')

			meshA = sel[0]
			meshB = sel[1]

			# Check that the meshes are not already involved in an operation
			for mesh in sel:
				attr = cmds.listAttr(mesh, ud=True)
				# If we fint attributes to check
				if attr:
					if gVar.outMeshToCompound in attr:
						result = cmds.confirmDialog( title='Nested network', message='%s appears to be the input mesh of another HM Operation\nDo you want to use it for a new HM Operation too?' % mesh, button=['Yes','No'], defaultButton='Yes', cancelButton='No', dismissString='No' )
						if result == 'No':
								createOperation = False
	  
	  	# Proceed in the creation only if the var is true
		if createOperation:
			#==== Get the interface values
			self.getValuesFromInterface()

			
			#==== Create the node network
			hmSetup = hmNodesConnection.HmOperationConnections()
			
			hmSetup.hmCompoundSetup( meshA, meshB, self.hmOpt_OperationVal[1], self.hmOpt_SmoothValA[1], self.hmOpt_SmoothValB[1], self.hmOpt_offsetVal[1], self.hmOpt_offsetVal[1], self.hmOpt_CornerBiasVal[1],
												self.hmOpt_GeoBiasVal[1], self.hmOpt_SectionsVal[1], self.hmOpt_RefineVal[1]-1, self.hmOpt_ChordDistVal[1],  gVar.uiFillStrategy[ self.hmOpt_StrategyVal[1] ], 
												self.hmOpt_SmoothProfileVal[1], self.hmOpt_WeightAVal[1], self.hmOpt_WeightBVal[1], self.hmOpt_ActiveRampedFillerVal[1], self.hmOpt_ProfileCurveVal[1], 
												self.hmOpt_RampChordVal[1], self.hmOpt_RampOffsetVal[1], self.hmOpt_RampScaleVal[1],  self.hmOpt_RampModeVal[1], self.hmOpt_BindBiasVal[1], self.hmOpt_RemoveCurlVal[1],
												self.hmOpt_ComputeNormalsVal[1], self.hmOpt_offsetType[1], self.hmOpt_ScaleFactor[1]) #We will use the arg 0 as shader

	def saveOptionsValues(self):
		'''
		This method stores the current options in the optionVar 
		'''
		#Get the interface values
		self.getValuesFromInterface()
		#Save them
		uiUtil.optionVarSave( self.optionsList )


	def resetOptionsValues(self):
		'''
		This method reset the options with the defualt values
		'''
		#Very important TRICK, let the gradient control forget about its option
		cmds.gradientControlNoAttr( self.gcProfileCurve, e=True, optionVar='' )
		
		
		
		#Reset the ui values
		if self.forceOperation == 0:
			cmds.optionMenu(self.omOperation, e=True, select=self.hmOpt_defaultOperation )
		
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value2= self.hmOpt_defaultSmoothB )
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value1= self.hmOpt_defaultSmoothA )
		
		if self.asParameterChanger == True:
			cmds.floatSliderGrp( self.fsOffsetA, e=True, v= self.hmOpt_defaultOffsetVal )
			cmds.floatSliderGrp( self.fsOffsetB, e=True, v= self.hmOpt_defaultOffsetVal )
		else:
			cmds.floatSliderGrp( self.fsOffset, e=True, v= self.hmOpt_defaultOffsetVal )
			
		cmds.floatSliderGrp( self.fsGlobCorBias, e=True, v= self.hmOpt_defaultCornerBias )
		cmds.floatSliderGrp( self.fsGlobGeoBias,  e=True, v= self.hmOpt_defaultGeoBias )
		cmds.intSliderGrp( self.isGlobSections, e=True, v= self.hmOpt_defaultSections )
		
		#Advanced		
		# cmds.checkBox( self.cbRefEdge, e=True, v=self.hmOpt_defaultRefine )
		cmds.optionMenuGrp( self.omRefEdge, e=True, select=self.hmOpt_defaultRefine )

		if gVar.globalScaleFactor == 1:
			cmds.floatSliderGrp( self.fsScaleFactor, e=True, v=self.hmOpt_defaultScaleFactor )


		cmds.floatSliderGrp( self.isAdaptVal, e=True, v=self.hmOpt_defaultChordDist )
		
		cmds.optionMenu( self.omStrategy, edit=True, select=self.hmOpt_defaultStrategy )
		
		cmds.checkBox( self.cbInterpProfile, e=True, v = self.hmOpt_defaultSmoothProfile )
		# cmds.checkBox( self.cbRefFiller, e=True, v = self.hmOpt_defaultFillerRefinment  ) ##########
		cmds.floatSliderGrp( self.fsWeightA, e=True, v = self.hmOpt_defaultWeightA )
		cmds.floatSliderGrp( self.fsWeightB, e=True, v = self.hmOpt_defaultWeightB )
		cmds.checkBox( self.cbActiveRamp,  e=True, v =self.hmOpt_defaultActiveRampedFiller )
		#Hide the ramp controls, since the default has the use ramp unactive
		self.disableProfileControl()
		

		#Set the gradient control to the preset n.1
		self.gFunct.setGradientControlTo1
		
		cmds.floatSliderGrp( self.fsRampChord, e=True, v=self.hmOpt_defaultRampChord )
		cmds.floatSliderGrp( self.fsRampOffset, e=True, v=self.hmOpt_defaultRampOffset)
		cmds.floatSliderGrp( self.fsRampScale, e=True, v=self.hmOpt_defaultRampScale)
		
		cmds.optionMenu(self.omInterp, e=True, select=self.hmOpt_defaultInterpolation )
		if gVar.develop == 1:
			# == Order To Split
			# cmds.floatField( self.ffBindBias, edit=True, v=self.hmOpt_defaultBindBias)
			# cmds.checkBox( self.cbSequencer, edit=True, v =self.hmOpt_defaultSequencer )
			# cmds.floatField( self.ffSeqTol, edit=True, v=self.hmOpt_defaultSequencerTolerance)
			cmds.checkBox( self.cbRemCurl, edit=True, v=self.hmOpt_defaultRemoveCurl)
			
			# == Again compound
			# cmds.checkBox( self.cbAutoMerge, e=True, v = self.hmOpt_defaultAutoMerge )
			# cmds.floatField( self.ffGlobTol, e=True, v= self.hmOpt_defaultTolerance )		
			cmds.checkBox( self.cbComputeNorm, e=True, v = self.hmOpt_defaultComputeNormals )
			cmds.checkBox( self.cbRampMode, e=True, v= self.hmOpt_defaultRampMode )

		
	def connectCompoundNode(self, inCompoundNode, inAuxNode):
		'''				
		This method queries the compound attributes, and connect them to current node		
		'''
		
		#Set the current compound for having right attributes names
		self.connectedCompound =  inCompoundNode
		self.connectedOrderToSplit = inAuxNode
		self.connectedOffsetNode = inAuxNode
		
		#Initialize the node attributes names
		self.initCompoundAttr()

		# ==== Operation
		cmds.connectControl(self.omOperation, self.nodeOperationType)
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value1= cmds.getAttr(self.nodeSmoothAttrNameA) )
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value2= cmds.getAttr(self.nodeSmoothAttrNameB) )
		
		# ==== Basics
		# This update the UI and connect the right controls
		self.updateOffsetStatus()

		cmds.connectControl(self.fsGlobCorBias, self.nodeCornerBias)
		cmds.connectControl(self.fsGlobGeoBias, self.nodeGeometryBias)
		cmds.connectControl(self.isGlobSections, self.nodeSectionSpans)
		
		# ==== Split Refinment
		cmds.connectControl(self.omRefEdge, self.nodeRefineEdge, index=2) # We need to use the index (the real control) or it wont work...

		if gVar.globalScaleFactor == 1:
			cmds.connectControl(self.fsScaleFactor, self.nodeOffsetScaleFactor )

		cmds.connectControl(self.isAdaptVal, self.nodeChordDist)

		#Activate or deactivate the chord dist control
		# self.setRefineOptStatus()


		cmds.connectControl(self.omStrategy, self.nodeFillStrategy)
		
		
		# ===== Interpolation and weights
		cmds.connectControl(self.cbInterpProfile, self.nodeSmoothFiller)
		
		
		# Activate or deactivate the weights
		self.setWeightsStatus()
		
		
		cmds.connectControl(self.fsWeightA, self.nodeWeightA)
		cmds.connectControl(self.fsWeightB, self.nodeWeightB)
		
		
		# ====== Ramped profile
		cmds.connectControl(self.cbActiveRamp, self.nodeUseRamp)
		
		#If the ramp is active, we need to make the tab visible, else to turn it off..
		if cmds.getAttr( self.nodeUseRamp ) == 1:
			self.enableProfileControl()
		else:
			self.disableProfileControl()
		
		#Connect the ramp chord, offset and scale controls
		
		cmds.connectControl(self.fsRampChord, self.nodeRampChord)
		cmds.connectControl(self.fsRampOffset, self.nodeRampOffset)
		cmds.connectControl(self.fsRampScale, self.nodeRampScale)
		
		#================================================ 
		#========= Sobstitute the current gradient control with a connected gradient control
		try:
			cmds.deleteUI(self.gcProfileCurve, self.omInterp, self.ffInterpPos, self.ffInterpVal, self.editGradButton )
		except:
			cmds.error('error in gradient control creation')


		# connect the gradient control for editing
		self.gFunct.setNode( self.connectedCompound)

		

		self.gcProfileCurve = cmds.gradientControl( at=self.nodeRampedFiller, h=90, w= 200, parent= self.gradientParent )	
		self.editGradButton = cmds.button(label= '>', h=90, w= 20, c=self.editGradientAttributeInWindow, visible = self.asParameterChanger, parent= self.gradientParent)

		#Create the auxiliary controls
		self.omInterp  = cmds.attrEnumOptionMenuGrp( label = '', w=60, columnWidth = [1,0], attribute=self.nodeRampedFiller_InterpONE, parent = self.gradientAttrParent)
		self.ffInterpPos  = cmds.attrFieldSliderGrp( label = '', w=80, columnWidth = [1,0], attribute=self.nodeRampedFiller_PosONE, parent = self.gradientAttrParent)
		self.ffInterpVal  = cmds.attrFieldSliderGrp( label = '', w=80, columnWidth = [1,0], attribute=self.nodeRampedFiller_ValONE, parent = self.gradientAttrParent)
		
		#Connect them
		cmds.connectControl( self.omInterp , self.nodeRampedFiller_InterpONE)
		cmds.connectControl( self.ffInterpPos, self.nodeRampedFiller_PosONE)
		cmds.connectControl( self.ffInterpVal, self.nodeRampedFiller_ValONE)
		
		#Tell the port about the controls, this will update the controls.. yhee.. fuck it
		cmds.gradientControl(self.gcProfileCurve, edit=True, sic = self.omInterp  )
		cmds.gradientControl(self.gcProfileCurve, edit=True, spc = self.ffInterpPos )
		cmds.gradientControl(self.gcProfileCurve, edit=True, scc = self.ffInterpVal )		
		
		
	
		
		if gVar.develop == 1:
			#================================================
			#Normals sliders, we just create 1-1 connection
			# cmds.connectControl(self.cbAutoMerge, self.nodeAutoMerge)
			# cmds.connectControl(self.ffGlobTol, self.nodeMergeDist)
			cmds.connectControl(self.cbRampMode, self.nodeRampMode)
			cmds.connectControl(self.cbComputeNorm, self.nodeComputeNormals)
			
			#================================================		
			# Order to Split Attributes
			if cmds.nodeType( inAuxNode ) == 'hmOrderToSplitNode':
				# cmds.connectControl(self.ffBindBias, self.nodeBindBias)
				# cmds.connectControl(self.cbSequencer, self.nodeCurveSequencer)
				# cmds.connectControl(self.ffSeqTol, self.nodeSequenceBias)
				cmds.connectControl(self.cbRemCurl, self.nodeRemoveCurl)
			
			#================================================		
			# Offset Curve Node Attributes
			if cmds.nodeType( inAuxNode ) == 'hmOffsetCurveNode':
				cmds.connectControl(self.cbRemCurl, self.nodeOffsetRemoveCurl)
				# cmds.connectControl(self.cbSequencer, self.nodeCurveSequencer)
				# cmds.connectControl(self.ffSeqTol, self.nodeSequenceBias)

			

			
	#===========================================================================================================  
	#================================ INTERFACE UPDATES ========================================================
	#===========================================================================================================
	def updateOffsetStatus(self):
		'''
		Display one or two offset sliders
		'''

		# Define the visibility of the controls
		if self.offsetIsSplitted():
			# print 'Dovrei avere DUE slider'
			cmds.columnLayout(self.doubleOffsetLayout, edit = True, visible = True)
			cmds.rowLayout( self.singleOffsetLayout, edit = True, visible = False)
			# Connect the controls
			cmds.connectControl(self.fsOffsetA, self.nodeOffsetAttrNameA)
			cmds.connectControl(self.fsOffsetB, self.nodeOffsetAttrNameB)
			
		else:
			# print 'Dovrei avere UNO slider'
			cmds.columnLayout(self.doubleOffsetLayout, edit = True, visible = False)
			cmds.rowLayout( self.singleOffsetLayout, edit = True, visible = True)
			# Connect the controls
			cmds.connectControl(self.fsOffset, self.nodeOffsetAttrNameA)

	def updateScaleOnCompound(self, *Args):
		'''
		Trick for connecting the same value to different maya nodes...
		'''
		if self.asParameterChanger:
			#if self.connectedCompound != None:
			mainVal = cmds.floatSliderGrp( self.fsScaleFactor, query = True, value = True )
			
			# Update the chord dist with the same value of the scale if we have the option selected
			# cmds.setAttr( self.nodeCompoundScaleFactor, mainVal ) SCALE ON COMPOUND NOT PRESENT ANYMORE

			# If the option is set on scale, we set the chord dist with the same value of scale
			if cmds.optionMenuGrp( self.omRefEdge, query=True, select=True ) == 4:
				cmds.setAttr( self.nodeChordDist, mainVal )

	def enableSeqTol(self, *Args ):
		cmds.floatField( self.ffSeqTol, edit=True, en=True )	
	def disableSeqTol(self, *Args ):
		cmds.floatField( self.ffSeqTol, edit=True, en=False )
		

	def enableProfileControl(self, *Args):
		#Make the ramp controls visible
		cmds.frameLayout( self.gradFrame, edit=True, visible=True )
		#Disable the sections spans
		#cmds.intSliderGrp( self.isGlobSections,  edit=True, en=False)
	def disableProfileControl(self, *Args):
		#Make the ramp controls invisible
		cmds.frameLayout( self.gradFrame, edit=True, visible=False )
		#Enable the sections spans
		#cmds.intSliderGrp( self.isGlobSections,  edit=True, en=True)

	# ======================== Chord Dist on off ==========================
	def setRefineOptStatus(self, *Args):
		'''
		Activate the chord distance only when refine is on Chord Dist
		'''

		valueOfRefine = cmds.optionMenuGrp( self.omRefEdge, query=True, select=True )


		# # If the value is set to on scale
		# if valueOfRefine == 4:
		# 	valueToSet = cmds.floatSliderGrp( self.fsScaleFactor, query = True, value = True )

		# 	# Update the chord dist with the same value of the scale if we have the option selected
		# 	if self.asParameterChanger:
		# 		if self.connectedCompound != None:
		# 			# For avoiding useless re computing of the graph, make sure 
		# 			# to set the attribute only if it's different from the scale parameter.
		# 			currentVal = cmds.getAttr(self.nodeChordDist)
		# 			if currentVal != valueToSet:
		# 				cmds.setAttr( self.nodeChordDist, valueToSet )

		# 	# Disable the chord dist slider
		# 	self.disableRefineOpt()
		# # If the value i set to chord dist, we need the slider, activate it
		# elif valueOfRefine == 2:
		# 	self.enableRefineOpt()
		# else:
		# 	self.disableRefineOpt()

			
	def enableRefineOpt(self, *Args ):
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=True ) 	
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, vis=True ) 	
	def disableRefineOpt(self, *Args ):
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=False ) 
		cmds.floatSliderGrp( self.isAdaptVal, edit=True, vis=False ) 
		
	# ======================== Weight AB on off ==========================
	def setWeightsStatus(self, *Args):
		valOfInterpolation = cmds.checkBox( self.cbInterpProfile, q=True, v = True )
		
		if valOfInterpolation == 0:
			cmds.floatSliderGrp( self.fsWeightA, e=True, vis = False )
			cmds.floatSliderGrp( self.fsWeightB, e=True, vis = False )
		else:
			cmds.floatSliderGrp( self.fsWeightA, e=True, vis = True )
			cmds.floatSliderGrp( self.fsWeightB, e=True, vis = True )
			
			
	
	def controlRampStatus(self):
		'''
		Since opening and closing the frame layout ignore the status of the control
		making it always visible, we have to check it again 
		'''
		#Get the current check box value...
		isOnOrOff = cmds.checkBox( self.cbActiveRamp, q=True, v = True )
		#But if its a par changer, get the attribute...
		if self.asParameterChanger:
			isOnOrOff = cmds.getAttr( self.nodeUseRamp )
		
		# Now we know if when opening the layout the control must be visible or not
		if isOnOrOff:
			self.enableProfileControl()
		else:
			self.disableProfileControl()
			
			

	
	#===========================================================================================================
	#================================ PARAMETER CHANGER COMMANDS ========================================================	
	#===========================================================================================================
	def updateSmoothUI(self, *Args):
		# update UI
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value1= cmds.getAttr(self.nodeSmoothAttrNameA) )
		cmds.intFieldGrp( self.ifgSmoothAB, e=True, value2= cmds.getAttr(self.nodeSmoothAttrNameB) )

		
	def changeSmoothValues(self, *Args):
		'''
		This method sets the smooth values of the compound node.
		Because seems to be very slow, we put it here.
		'''
		#Get value from UI
		self.getValuesFromInterface()			

		#Set the attribute to the connected meshes
		cmds.setAttr( '%s.%s' % (self.meshA, gVar.smoothAttrName), self.hmOpt_SmoothValA[1])
		cmds.setAttr( '%s.%s' % (self.meshB, gVar.smoothAttrName), self.hmOpt_SmoothValB[1])

			
	def editGradientAttributeInWindow(self, *Args):
		'''
		Edit the current connected attribute in the standard maya window 
		'''
		mel.eval( 'editRampAttribute %s' % self.nodeRampedFiller)
	
	
	
	
	
	#===========================================================================================================
	#================================ UTILITY ========================================================
	#===========================================================================================================

	def offsetIsSplitted(self):
		'''
		This method returns True or False if depending on the connection
		of the current offset node attribute offsetB to the offsetA
		'''
		if self.connectedOffsetNode:
			connected = cmds.listConnections('%s.%s' % (self.connectedOffsetNode, gVar.offsetB), p=True)
			#if there is a connection
			if connected:
			    if connected[0].split('.')[1] == gVar.offsetA:
			    	return False
			    else:
			    	return True
			else:
				return True		

	def splitOffset(self, *Args):
		'''
		This is called for splitting the current offset
		'''	

		# Break the connection
		driver = '%s.%s' % (self.connectedOffsetNode, gVar.offsetA)
		driven = '%s.%s' % (self.connectedOffsetNode, gVar.offsetB)

		cmds.disconnectAttr(driver, driven)

		# Update UI
		self.updateOffsetStatus()



	def mergeOffset(self, *Args):
		'''
		This method is called for merging back the two offset values
		'''
		# Create the connection
		driver = '%s.%s' % (self.connectedOffsetNode, gVar.offsetA)
		driven = '%s.%s' % (self.connectedOffsetNode, gVar.offsetB)

		cmds.connectAttr(driver, driven)

		# Update UI
		self.updateOffsetStatus()

	def getValuesFromInterface(self, *Args):
		'''
		This method store in the main variables the values of the interface
		'''
		self.hmOpt_OperationVal[1] = cmds.optionMenu(self.omOperation, q=True, select=True )
		self.hmOpt_SmoothValA[1] =cmds.intFieldGrp( self.ifgSmoothAB, q=True, value1 = True )
		self.hmOpt_SmoothValB[1] =cmds.intFieldGrp( self.ifgSmoothAB, q=True, value2 = True )
		
		if self.asParameterChanger == True:
			self.hmOpt_offsetValA  = cmds.floatSliderGrp( self.fsOffsetA, q=True, v=True)
			self.hmOpt_offsetValB  = cmds.floatSliderGrp( self.fsOffsetB, q=True, v=True)
		else:
			self.hmOpt_offsetVal[1]  = cmds.floatSliderGrp( self.fsOffset, q=True, v=True)
		
		self.hmOpt_CornerBiasVal[1] = cmds.floatSliderGrp( self.fsGlobCorBias, q=True, v=True)
		self.hmOpt_GeoBiasVal[1] =  cmds.floatSliderGrp( self.fsGlobGeoBias,  q=True, v=True)
		self.hmOpt_SectionsVal[1] = cmds.intSliderGrp( self.isGlobSections, q=True, v=True)

		#Advanced
		# self.hmOpt_RefineVal[1] = cmds.checkBox( self.cbRefEdge, query=True, v=True )
		self.hmOpt_RefineVal[1] = cmds.optionMenuGrp( self.omRefEdge, query=True, select=True ) 

		# If we are using the global scale factor
		if gVar.globalScaleFactor == 1:
			# Get its value
			self.hmOpt_ScaleFactor[1] = cmds.floatSliderGrp( self.fsScaleFactor, query=True, v=True )
			# If we are in the mode "on scale", the chord dist value that we get must be equal to the chord dist.
			if self.hmOpt_RefineVal[1] == 4:
				self.hmOpt_ChordDistVal[1] = self.hmOpt_ScaleFactor[1]
			# Otherwise it must be readed from the ui value
			else:
				self.hmOpt_ChordDistVal[1] = cmds.floatSliderGrp( self.isAdaptVal, query=True, v=True )
		
		else:
			self.hmOpt_ChordDistVal[1] = cmds.floatSliderGrp( self.isAdaptVal, query=True, v=True )

		self.hmOpt_StrategyVal[1] =  cmds.optionMenu( self.omStrategy, q=True, select=True )

			
		
		self.hmOpt_SmoothProfileVal[1] = cmds.checkBox( self.cbInterpProfile, q=True, v = True )
		# self.hmOpt_FillerRefinmentVal[1] =  cmds.checkBox( self.cbRefFiller, q=True, v = True )
		self.hmOpt_WeightAVal[1] = cmds.floatSliderGrp( self.fsWeightA, q=True, v = True )
		self.hmOpt_WeightBVal[1] = cmds.floatSliderGrp( self.fsWeightB, q=True, v = True )
		self.hmOpt_ActiveRampedFillerVal[1] = cmds.checkBox( self.cbActiveRamp, q=True, v = True )
		#Query the profile curve with no attr only if the case
		if self.asParameterChanger == False:
			# cmds.gradientControlNoAttr( self.gcProfileCurve, q=True, optionVar=True ) This only returns the optionVar name
			self.hmOpt_ProfileCurveVal[1] = cmds.optionVar( q=self.hmOpt_ProfileCurveVal[0])
			self.hmOpt_InterpolationVal[1] = cmds.optionMenu(self.omInterp, q=True, select=True )

		self.hmOpt_RampOffsetVal[1] =  cmds.floatSliderGrp( self.fsRampOffset, q=True, v=True)
		self.hmOpt_RampScaleVal[1] =  cmds.floatSliderGrp( self.fsRampScale, q=True, v=True)
		
		if gVar.develop == 1:
			# == Order To Split Parameters
			# self.hmOpt_BindBiasVal[1] = cmds.floatField( self.ffBindBias, q=True, v=True)
			# self.hmOpt_SequencerVal[1] = cmds.checkBox( self.cbSequencer, query=True, v =True )
			# self.hmOpt_SequencerToleranceVal[1] = cmds.floatField( self.ffSeqTol, q=True, v=True)
			self.hmOpt_RemoveCurlVal[1] = cmds.checkBox( self.cbRemCurl, q=True, v=True)
			
			# == Again compound
			# self.hmOpt_AutoMergeVal[1] = cmds.checkBox( self.cbAutoMerge, query=True, v =True )
			# self.hmOpt_ToleranceVal[1] = cmds.floatField( self.ffGlobTol, q=True, v=True)
			self.hmOpt_RampChordVal[1] = cmds.floatSliderGrp( self.fsRampChord, q=True, v=True )
			self.hmOpt_ComputeNormalsVal[1] = cmds.checkBox( self.cbComputeNorm, q=True, v = True )
			self.hmOpt_RampModeVal[1] =  cmds.checkBox(self.cbRampMode, q=True, v=True )
		
		# ====== DEVELOPING VAR 
		if gVar.develop == 1 and self.asParameterChanger == False:
			self.hmOpt_offsetType[1] = cmds.radioButtonGrp( self.offTypeOperation, q=True, select=True ) 
			
			

	def getMainTooltip(self):
		'''
		This method return the main tooltip hint for adding it to the main button
		'''
		return self.tooltipArray['Main Click']
